import contextlib

import nonebot

nonebot.load_plugin("nonebot_plugin_alconna.uniseg")

from tarina import lang
from nonebot import get_plugin_config
from arclet.alconna import Args as Args
from arclet.alconna import Field as Field
from arclet.alconna import count as count
from nonebot.plugin import PluginMetadata
from arclet.alconna import Option as Option
from arclet.alconna import append as append
from arclet.alconna import config as config
from arclet.alconna import Alconna as Alconna
from arclet.alconna import Arparma as Arparma
from arclet.alconna import ArgsStub as ArgsStub
from arclet.alconna import MultiVar as MultiVar
from arclet.alconna import Namespace as Namespace
from arclet.alconna import namespace as namespace
from arclet.alconna import KeyWordVar as KeyWordVar
from arclet.alconna import OptionStub as OptionStub
from arclet.alconna import Subcommand as Subcommand
from arclet.alconna import store_true as store_true
from arclet.alconna import CommandMeta as CommandMeta
from arclet.alconna import Duplication as Duplication
from arclet.alconna import store_false as store_false
from arclet.alconna import store_value as store_value
from arclet.alconna import OptionResult as OptionResult
from arclet.alconna import append_value as append_value
from arclet.alconna import SubcommandStub as SubcommandStub
from arclet.alconna import output_manager as output_manager
from arclet.alconna import ArparmaBehavior as ArparmaBehavior
from arclet.alconna import command_manager as command_manager
from arclet.alconna import SubcommandResult as SubcommandResult

from .consts import log
from .config import Config
from .uniseg import At as At
from .uniseg import Card as Card
from .uniseg import File as File
from .uniseg import Text as Text
from .model import Match as Match
from .model import Query as Query
from .params import Check as Check
from .uniseg import AtAll as AtAll
from .uniseg import Audio as Audio
from .uniseg import Emoji as Emoji
from .uniseg import Image as Image
from .uniseg import MsgId as MsgId
from .uniseg import Other as Other
from .uniseg import Reply as Reply
from .uniseg import Video as Video
from .uniseg import Voice as Voice
from .params import assign as assign
from .rule import alconna as alconna
from .uniseg import Custom as Custom
from .uniseg import Target as Target
from .uniseg import UniMsg as UniMsg
from .uniseg import RefNode as RefNode
from .uniseg import Segment as Segment
from .uniseg import get_bot as get_bot
from .matcher import Command as Command
from .matcher import referent as referent
from .params import AlcResult as AlcResult
from .uniseg import MessageId as MessageId
from .uniseg import MsgTarget as MsgTarget
from .uniseg import Reference as Reference
from .uniseg import patch_saa as patch_saa
from .argv import MessageArgv as MessageArgv
from .params import AlcContext as AlcContext
from .params import AlcMatches as AlcMatches
from .params import AlconnaArg as AlconnaArg
from .params import match_path as match_path
from .uniseg import CustomNode as CustomNode
from .uniseg import UniMessage as UniMessage
from .extension import Extension as Extension
from .extension import Interface as Interface
from .matcher import funcommand as funcommand
from .matcher import on_alconna as on_alconna
from .params import match_value as match_value
from .uniseg import image_fetch as image_fetch
from .params import AlconnaMatch as AlconnaMatch
from .params import AlconnaQuery as AlconnaQuery
from .model import CommandResult as CommandResult
from .params import AlcExecResult as AlcExecResult
from .params import AlconnaResult as AlconnaResult
from .uniseg import MessageTarget as MessageTarget
from .consts import ALCONNA_RESULT as ALCONNA_RESULT
from .params import AlconnaContext as AlconnaContext
from .params import AlconnaMatches as AlconnaMatches
from .uniseg import SupportAdapter as SupportAdapter
from .matcher import AlconnaMatcher as AlconnaMatcher
from .consts import ALCONNA_ARG_KEY as ALCONNA_ARG_KEY
from .uniseg import SerializeFailed as SerializeFailed
from .uniseg import custom_register as custom_register
from .extension import load_from_path as load_from_path
from .uniseg import UniversalMessage as UniversalMessage
from .uniseg import UniversalSegment as UniversalSegment
from .params import AlconnaExecResult as AlconnaExecResult
from .params import AlconnaDuplication as AlconnaDuplication
from .consts import ALCONNA_EXEC_RESULT as ALCONNA_EXEC_RESULT
from .extension import add_global_extension as add_global_extension

__version__ = "0.38.1"

__plugin_meta__ = PluginMetadata(
    name="Alconna 插件",
    description="提供 ArcletProject/Alconna 的 Nonebot2 适配版本与工具",
    usage="matcher = on_alconna(...)",
    homepage="https://github.com/nonebot/plugin-alconna",
    type="library",
    supported_adapters=None,
    config=Config,
    extra={
        "author": "RF-Tar-Railt",
        "priority": 1,
        "version": __version__,
    },
)

with contextlib.suppress(ValueError, LookupError):
    _config = get_plugin_config(Config)
    for path in _config.alconna_global_extensions:
        log("DEBUG", lang.require("nbp-alc", "log.load_global_extensions").format(path=path))
        load_from_path(path)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union, cast

from tarina import lang
from nonebot.adapters import Bot, Event, Message

from ..segment import At, File, Text, Audio, Emoji, Image, Reply, Video, Voice
from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export

if TYPE_CHECKING:
    from nonebot.adapters.telegram.message import MessageSegment


class TelegramMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.telegram.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.telegram

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.telegram.event import EventWithChat

        assert isinstance(event, EventWithChat)
        return Target(str(event.chat.id), platform=self.get_adapter(), self_id=bot.self_id if bot else None)

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.telegram.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return f"{event.message_id}"

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.telegram.message import Entity

        if not seg.style:
            return Entity.text(seg.text)
        else:
            return Entity(seg.style, {"text": seg.text})

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.telegram.message import Entity

        return (
            Entity.mention(f"{seg.target} ")
            if seg.target.startswith("@")
            else Entity.text_link("用户 ", f"tg://user?id={seg.target}")
        )

    @export
    async def emoji(self, seg: Emoji, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.telegram.message import Entity

        return Entity.custom_emoji(seg.name, seg.id)  # type: ignore

    @export
    async def media(self, seg: Union[Image, Voice, Video, Audio, File], bot: Bot) -> "MessageSegment":
        from nonebot.adapters.telegram.message import File as TgFile

        name = seg.__class__.__name__.lower()
        method = {
            "image": TgFile.photo,
            "voice": TgFile.voice,
            "video": TgFile.video,
            "audio": TgFile.audio,
            "file": TgFile.document,
        }[name]
        if seg.id or seg.url:
            return method(seg.id or seg.url)
        elif seg.path:
            return method(Path(seg.path).read_bytes())
        elif seg.raw:
            return method(seg.raw_bytes)
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.telegram.message import Reply as TgReply

        return TgReply.reply(int(seg.id))

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.telegram.bot import Bot as TgBot
        from nonebot.adapters.telegram.event import Event as TgEvent
        from nonebot.adapters.telegram.message import Message as TgMessage

        assert isinstance(bot, TgBot)
        assert isinstance(message, TgMessage)
        if isinstance(target, Event):
            assert isinstance(target, TgEvent)
            return await bot.send(event=target, message=message)
        return await bot.send_to(target.id, message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.telegram.bot import Bot as TgBot
        from nonebot.adapters.telegram.model import Message as MessageModel

        assert isinstance(bot, TgBot)
        _mid: MessageModel = cast(MessageModel, mid)
        await bot.delete_message(chat_id=_mid.chat.id, message_id=_mid.message_id)

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.telegram.bot import Bot as TgBot
        from nonebot.adapters.telegram.model import Message as MessageModel

        assert isinstance(bot, TgBot)
        _mid: MessageModel = cast(MessageModel, mid)
        text = new.extract_plain_text()
        res = await bot.edit_message_text(text=text, chat_id=_mid.chat.id, message_id=_mid.message_id)
        if isinstance(res, MessageModel):
            return res

    def get_reply(self, mid: Any):
        from nonebot.adapters.telegram.model import Message as MessageModel

        _mid: MessageModel = cast(MessageModel, mid)
        return Reply(str(_mid.message_id))



from typing import TYPE_CHECKING, Union

from tarina import lang
from nonebot.adapters import Bot, Event, Message

from ..segment import Card, File, Text, Image, Video
from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export

if TYPE_CHECKING:
    from nonebot.adapters.ntchat.message import MessageSegment


class NTChatMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.ntchat.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.ntchat

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from_wxid = getattr(event, "from_wxid", None)
        room_wxid = getattr(event, "room_wxid", "")
        if from_wxid:
            return Target(from_wxid, room_wxid, platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        raise NotImplementedError

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.ntchat.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.msgid)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.text(seg.text)

    @export
    async def res(self, seg: Union[Image, File, Video], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        method = {
            "image": ms.image,
            "video": ms.video,
            "file": ms.file,
        }[name]
        if seg.path:
            return method(seg.path)
        elif seg.raw:
            return method(seg.raw_bytes)
        elif seg.url or seg.id:
            return method(seg.url or seg.id)  # type: ignore
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def card(self, seg: Card, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.flag != "xml":
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="card", seg=seg))

        return ms.xml(seg.raw)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.ntchat.bot import send
        from nonebot.adapters.ntchat.bot import Bot as NTChatBot

        assert isinstance(bot, NTChatBot)

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        class FakeEvent:
            from_wxid = target.id
            if target.parent_id:
                room_wxid = target.parent_id

        return await send(bot, FakeEvent, message)  # type: ignore



import pytest
from nonebug import App
from nonebot import get_adapter
from arclet.alconna import Arparma
from nonebot.adapters.onebot.v11 import Bot, Adapter, Message

from tests.fake import fake_group_message_event_v11


@pytest.mark.asyncio()
async def test_ref(app: App):
    from nonebot_plugin_alconna import Command, referent

    book = (
        Command("book", "测试")
        .option("writer", "-w <id:int>")
        .option("writer", "--anonymous", {"id": 0})
        .usage("book [-w <id:int> | --anonymous]")
        .shortcut("测试", {"args": ["--anonymous"]})
        .build()
    )

    @book.handle()
    async def _(arp: Arparma):
        await book.send(f"0: {(arp.options)}")

    async with app.test_matcher(book) as ctx:  # type: ignore
        adapter: Adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter)
        event = fake_group_message_event_v11(message=Message("book --anonymous"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, "0: {'writer': (value=Ellipsis args={'id': 0})}")

    book1 = referent(book.command)
    assert id(book1) == id(book)

    @book1.handle(override=("replace", 0))
    async def _(arp: Arparma):
        await book1.send(f"1: {(arp.options)}")

    @book1.handle(override=("insert", 0))
    async def _(arp: Arparma):
        await book1.send(f"2: {(arp.options)}")

    async with app.test_matcher(book) as ctx:  # type: ignore
        adapter: Adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter)
        event = fake_group_message_event_v11(message=Message("book --anonymous"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, "2: {'writer': (value=Ellipsis args={'id': 0})}")
        ctx.should_call_send(event, "1: {'writer': (value=Ellipsis args={'id': 0})}")



import re
import functools
from string import Formatter
from typing_extensions import TypeAlias
from typing import (
    TYPE_CHECKING,
    Any,
    Set,
    Dict,
    Type,
    Tuple,
    Union,
    Mapping,
    TypeVar,
    Callable,
    Optional,
    Sequence,
    cast,
)

import _string  # type: ignore
from tarina.tools import gen_subclass

from .segment import Segment

if TYPE_CHECKING:
    from .message import UniMessage

FormatSpecFunc: TypeAlias = Callable[[Any], str]
FormatSpecFunc_T = TypeVar("FormatSpecFunc_T", bound=FormatSpecFunc)

_MAPPING = {cls.__name__: cls for cls in gen_subclass(Segment)}
_PATTERN = re.compile("(" + "|".join(_MAPPING.keys()) + r")\((.*)\)$")


def _eval(route: str, obj: Any):
    res = obj
    parts = re.split(r"\.|(\[.+\])|(\(.*\))", route)
    for part in parts[1:]:
        if not part:
            continue
        if part.startswith("_"):
            raise ValueError(route)
        if part.startswith("[") and part.endswith("]"):
            item = part[1:-1]
            if item[0] in ("'", '"') and item[-1] in ("'", '"'):
                res = res[item[1:-1]]
            elif ":" in item:
                res = res[slice(*(int(x) if x else None for x in item.split(":")))]
            else:
                res = res[int(item)]
        elif part.startswith("(") and part.endswith(")"):
            item = part[1:-1]
            if not item:
                res = res()
            else:
                _parts = item.split(",")
                _args = []
                _kwargs = {}
                for part in _parts:
                    part = part.strip()
                    if re.match(".+=.+", part):
                        k, v = part.split("=")
                        _kwargs[k] = v
                    else:
                        _args.append(part)
                res = res(*_args, **_kwargs)
        else:
            res = getattr(res, part)
    return res


class UniMessageTemplate(Formatter):
    """通用消息模板格式化实现类。

    参数:
        template: 模板
        factory: 消息类型工厂，默认为 `str`
    """

    def __init__(self, template: Union[str, "UniMessage"], factory: Type["UniMessage"]) -> None:
        self.template = template
        self.factory = factory
        self.format_specs: Dict[str, FormatSpecFunc] = {}

    def __repr__(self) -> str:
        return f"UniMessageTemplate({self.template!r})"

    def add_format_spec(self, spec: FormatSpecFunc_T, name: Optional[str] = None) -> FormatSpecFunc_T:
        name = name or spec.__name__
        if name in self.format_specs:
            raise ValueError(f"Format spec {name} already exists!")
        self.format_specs[name] = spec
        return spec

    def format(self, *args, **kwargs):
        """根据传入参数和模板生成消息对象"""
        return self._format(args, kwargs)

    def format_map(self, mapping: Mapping[str, Any]):
        """根据传入字典和模板生成消息对象, 在传入字段名不是有效标识符时有用"""
        return self._format([], mapping)

    def _format(self, args: Sequence[Any], kwargs: Mapping[str, Any]):
        full_message = self.factory()
        used_args, arg_index = set(), 0

        if isinstance(self.template, str):
            msg, arg_index = self._vformat(self.template, args, kwargs, used_args, arg_index)
            full_message += msg
        elif isinstance(self.template, self.factory):
            template = cast("UniMessage[Segment]", self.template)
            for seg in template:
                if not seg.is_text():
                    full_message += seg
                else:
                    msg, arg_index = self._vformat(str(seg), args, kwargs, used_args, arg_index)
                    full_message += msg
        else:
            raise TypeError("template must be a string or instance of UniMessage!")

        # self.check_unused_args(used_args, args, kwargs)
        return full_message

    # def check_unused_args(
    #     self,
    #     used_args: Sequence[Union[str, int]],
    #     args: Sequence[Any],
    #     kwargs: Mapping[str, Any],
    # ) -> None:
    #     indexes = [i for i in used_args if isinstance(i, int)]
    #     if indexes and len(indexes) != len(args):
    #         raise ValueError(
    #             f"not all arguments converted during string formatting: "
    #             f"{[c for i, c in enumerate(args) if i not in indexes]}"
    #         )
    #     keys = [k for k in used_args if isinstance(k, str)]
    #     if keys and keys != list(kwargs.keys()):
    #         raise ValueError(
    #             f"not all arguments converted during string formatting: " f"{set(kwargs) - set(keys)}"
    #         )

    def _vformat(
        self,
        format_string: str,
        args: Sequence[Any],
        kwargs: Mapping[str, Any],
        used_args: Set[Union[int, str]],
        auto_arg_index: int = 0,
    ) -> Tuple["UniMessage", int]:
        results: list = [self.factory()]

        for literal_text, field_name, format_spec, conversion in self.parse(format_string):
            # output the literal text
            if literal_text:
                results.append(literal_text)

            # if there's a field, output it
            if field_name is not None:
                if field_name == "" and format_spec and (mat := _PATTERN.match(format_spec)):
                    cls, parts = _MAPPING[mat[1]], mat[2].split(",")
                    _args = []
                    _kwargs = {}
                    for part in parts:
                        part = part.strip()
                        if part.startswith("$") and (key := part.split(".")[0]) in kwargs:
                            _args.append(_eval(part[1:], kwargs[key]))
                        elif re.match(".+=.+", part):
                            k, v = part.split("=")
                            if v in kwargs:
                                _kwargs[k] = kwargs[v]
                                used_args.add(v)
                            elif v.startswith("$") and (key := v.split(".")[0]) in kwargs:
                                _kwargs[k] = _eval(v[1:], kwargs[key])
                            else:
                                _kwargs[k] = v
                        elif part in kwargs:
                            _args.append(kwargs[part])
                            used_args.add(part)
                        else:
                            _args.append(part)
                    results.append(cls(*_args, **_kwargs))  # type: ignore
                    continue
                if field_name == "":
                    if auto_arg_index is False:
                        raise ValueError(
                            "cannot switch from manual field specification to " "automatic field numbering"
                        )
                    field_name = str(auto_arg_index)
                    auto_arg_index += 1
                elif field_name.isdigit():
                    if auto_arg_index:
                        raise ValueError(
                            "cannot switch from manual field specification to " "automatic field numbering"
                        )
                    # disable auto arg incrementing, if it gets
                    # used later on, then an exception will be raised
                    auto_arg_index = False

                # given the field_name, find the object it references
                #  and the argument it came from
                obj, arg_used = self.get_field(field_name, args, kwargs)
                used_args.add(arg_used)

                # do any conversion on the resulting object
                obj = self.convert_field(obj, conversion) if conversion else obj

                # format the object and append to the result
                formatted_text = self.format_field(obj, format_spec) if format_spec else obj
                results.append(formatted_text)

        return functools.reduce(self._add, results), auto_arg_index

    def format_field(self, value: Any, format_spec: str) -> Any:
        formatter: Optional[FormatSpecFunc] = self.format_specs.get(format_spec)
        if formatter is None and format_spec in _MAPPING:
            formatter = _MAPPING[format_spec]  # type: ignore
        return super().format_field(value, format_spec) if formatter is None else formatter(value)

    def get_field(self, field_name, args, kwargs):
        first, rest = _string.formatter_field_name_split(field_name)

        obj = self.get_value(first, args, kwargs)

        return obj, first

    def _add(self, a: Any, b: Any) -> Any:
        try:
            return a + b
        except TypeError:
            return a + str(b)



from typing import TYPE_CHECKING, Union

from nonebot.adapters import Bot, Event, Message

from ..segment import At, Text, AtAll, Image
from ..export import Target, SupportAdapter, MessageExporter, export

if TYPE_CHECKING:
    from nonebot.adapters.ding.message import MessageSegment


class DingMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.ding.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.ding

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.ding.event import MessageEvent, ConversationType

        if isinstance(event, MessageEvent):
            if event.conversationType == ConversationType.private:
                return Target(
                    event.senderId,
                    private=True,
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
            return Target(event.conversationId, platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        raise NotImplementedError

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.ding.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.msgId)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.atDingtalkIds(seg.target)

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.atAll()

    @export
    async def image(self, seg: Image, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        assert seg.url, "ding image segment must have url"
        return ms.image(seg.url)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        raise NotImplementedError



from typing import TYPE_CHECKING, Union

from nonebot.adapters import Bot, Event, Message

from ..segment import Text, Emoji
from ..export import Target, SupportAdapter, MessageExporter, export

if TYPE_CHECKING:
    from nonebot.adapters.console.message import MessageSegment


class ConsoleMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.console.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.console

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.console.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.self_id)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        if seg.style and seg.style.startswith("markup"):
            _style = seg.style.split(":", 1)[-1]
            return ms.markup(seg.text, _style)
        if seg.style and seg.style.startswith("markdown"):
            code_theme = seg.style.split(":", 1)[-1]
            return ms.markdown(seg.text, code_theme)
        return ms.text(seg.text)

    @export
    async def emoji(self, seg: Emoji, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.emoji(seg.name or seg.id)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.console import Bot as ConsoleBot

        assert isinstance(bot, ConsoleBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())
        if isinstance(target, Event):
            target = self.get_target(target, bot)
        return await bot.send_msg(user_id=target.id, message=message)



from typing import Union

from nepattern.base import INTEGER
from nepattern import MatchMode, BasePattern, UnionPattern
from nonebot.adapters.feishu.message import MessageSegment

from nonebot_plugin_alconna.typings import SegmentPattern

Text = str
At = SegmentPattern("at", MessageSegment, MessageSegment.at, additional=lambda x: x.data["user_id"] != "all")
AtAll = SegmentPattern(
    "at", MessageSegment, lambda: MessageSegment.at("all"), additional=lambda x: x.data["user_id"] == "all"
)
Post = SegmentPattern("post", MessageSegment, MessageSegment.post)
Image = SegmentPattern("image", MessageSegment, MessageSegment.image)
Interactive = SegmentPattern("interactive", MessageSegment, MessageSegment.interactive)
ShareChat = SegmentPattern("share_chat", MessageSegment, MessageSegment.share_chat)
ShareUser = SegmentPattern("share_user", MessageSegment, MessageSegment.share_user)
Audio = SegmentPattern("audio", MessageSegment, MessageSegment.audio)
Video = Media = SegmentPattern("media", MessageSegment, MessageSegment.media)
File = SegmentPattern("File", MessageSegment, MessageSegment.file)
Sticker = SegmentPattern("sticker", MessageSegment, MessageSegment.sticker)


AtID = (
    UnionPattern[Union[str, MessageSegment]](
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=int,
                alias="At",
                addition_accepts=At,
                converter=lambda _, x: int(x.data["user_id"]),
            ),
            BasePattern(
                r"@(\d+)",
                mode=MatchMode.REGEX_CONVERT,
                origin=int,
                alias="@xxx",
                accepts=str,
                converter=lambda _, x: int(x[1]),
            ),
            INTEGER,
        ]
    )
    @ "at_id"
)
"""
内置类型，允许传入提醒元素(At)或者'@xxxx'式样的字符串或者数字, 返回数字
"""



import inspect
from enum import Enum
from abc import ABCMeta, abstractmethod
from dataclasses import field, dataclass
from typing import TYPE_CHECKING, Any, Dict, Type, Union, Generic, TypeVar, Callable, Awaitable, get_args, get_origin

from tarina import lang
from nonebot.adapters import Bot, Event, Message, MessageSegment

from .segment import Other, Reply, Custom, Segment

if TYPE_CHECKING:
    from .message import UniMessage


TS = TypeVar("TS", bound=Segment)
TMS = TypeVar("TMS", bound=MessageSegment)


class SupportAdapter(str, Enum):
    """支持的适配器"""

    bilibili = "BilibiliLive"
    console = "Console"
    ding = "Ding"
    discord = "Discord"
    dodo = "DoDo"
    feishu = "Feishu"
    github = "GitHub"
    kook = "Kaiheila"
    minecraft = "Minecraft"
    mirai = "mirai2"
    ntchat = "ntchat"
    onebot11 = "OneBot V11"
    onebot12 = "OneBot V12"
    qq = "QQ"
    red = "RedProtocol"
    satori = "Satori"
    telegram = "Telegram"


@dataclass
class Target:
    id: str
    """目标id；若为群聊则为group_id或者channel_id，若为私聊则为user_id"""
    parent_id: str = ""
    """父级id；若为频道则为guild_id，其他情况为空字符串"""
    channel: bool = False
    """是否为频道，仅当目标平台符合频道概念时"""
    private: bool = False
    """是否为私聊"""
    source: str = ""
    """可能的事件id"""
    platform: Union[str, None] = None
    """平台(适配器)名称，若为None则需要明确指定 Bot 对象"""
    self_id: Union[str, None] = None
    """机器人id，若为None则需要明确指定 Bot 对象"""
    extra: Dict[str, Any] = field(default_factory=dict)
    """额外信息，用于适配器扩展"""

    async def send(
        self,
        message: Union[str, Message, "UniMessage"],
        bot: Union[Bot, None] = None,
        fallback: bool = True,
        at_sender: Union[str, bool] = False,
        reply_to: Union[str, bool, Reply, None] = False,
    ):
        """发送消息"""
        if isinstance(message, str):
            from .message import UniMessage

            message = UniMessage(message)
        if isinstance(message, Message):
            from .message import UniMessage

            message = await UniMessage.generate(message=message, bot=bot)
        return await message.send(self, bot, fallback, at_sender, reply_to)


class SerializeFailed(Exception): ...


def export(func: Callable[[Any, TS, Bot], Awaitable[MessageSegment]]):
    sig = inspect.signature(func)
    func.__export_target__ = sig.parameters["seg"].annotation
    return func


class MessageExporter(Generic[TMS], metaclass=ABCMeta):
    _mapping: Dict[Type[Segment], Callable[[Segment, Bot], Awaitable[MessageSegment]]]
    segment_class: Type[TMS]

    @classmethod
    @abstractmethod
    def get_adapter(cls) -> SupportAdapter: ...

    @abstractmethod
    def get_message_type(self) -> Type[Message]: ...

    @abstractmethod
    def get_message_id(self, event: Event) -> str: ...

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        return Target(event.get_user_id(), platform=self.get_adapter(), self_id=bot.self_id if bot else None)

    def __init__(self):
        self._mapping = {}
        for attr in self.__class__.__dict__.values():
            if callable(attr) and hasattr(attr, "__export_target__"):
                method = getattr(self, attr.__name__)
                target = attr.__export_target__
                if get_origin(target) is Union:
                    for t in get_args(target):
                        self._mapping[t] = method
                else:
                    self._mapping[target] = method

    async def export(self, source: "UniMessage", bot: Bot, fallback: bool):
        msg_type = self.get_message_type()
        message = msg_type()
        self.segment_class = msg_type.get_segment_class()
        for seg in source:
            seg_type = seg.__class__
            if seg_type in self._mapping:
                res = await self._mapping[seg_type](seg, bot)
                if isinstance(res, list):
                    message.extend(res)
                    break
                message.append(res)
            elif isinstance(seg, Custom):
                message.append(seg.export(msg_type))
            elif isinstance(seg, Other):
                message.append(seg.origin)  # type: ignore
            elif fallback:
                message += str(seg)
            else:
                raise SerializeFailed(
                    lang.require("nbp-uniseg", "failed").format(target=seg, adapter=bot.adapter.get_name())
                )
        return message

    @abstractmethod
    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        raise NotImplementedError

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        raise NotImplementedError

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        raise NotImplementedError

    def get_reply(self, mid: Any):
        raise NotImplementedError



from arclet.alconna import Args, Alconna


def test_kook():
    from nonebot.adapters.kaiheila.message import Message, MessageSegment

    from nonebot_plugin_alconna.adapters.kook import Mention, KMarkdown

    msg = (
        Message()
        + MessageSegment.text("/command ")
        + MessageSegment.mention("123456")
        + MessageSegment.KMarkdown("12345678")
    )

    alc = Alconna("/command", Args["some_arg", Mention]["some_arg1", KMarkdown])

    res = alc.parse(msg)
    assert res.matched
    assert res.some_arg.type == "mention"
    assert res.some_arg.data["user_id"] == "123456"
    assert res.some_arg1.data["content"] == "12345678"

    msg1 = Message([MessageSegment.text("/command1 "), MessageSegment.KMarkdown("[(met)123456(met)](42345) 12345678")])

    alc1 = Alconna("/command1", Args["some_arg", str]["some_arg1", str])

    res1 = alc1.parse(msg1)
    assert res1.matched
    assert res1.some_arg == "[(met)123456(met)](42345)"
    assert res1.some_arg1 == "12345678"



from typing import Union

from nepattern.base import INTEGER
from nonebot.adapters.red.message import MessageSegment
from nepattern import MatchMode, BasePattern, UnionPattern

from nonebot_plugin_alconna.typings import SegmentPattern

Text = str
At = SegmentPattern("at", MessageSegment, MessageSegment.at)
AtAll = SegmentPattern("at_all", MessageSegment, MessageSegment.at_all)
Face = SegmentPattern("face", MessageSegment, MessageSegment.face)
Image = SegmentPattern("image", MessageSegment, MessageSegment.image)
File = SegmentPattern("file", MessageSegment, MessageSegment.file)
Voice = SegmentPattern("voice", MessageSegment, MessageSegment.voice)
Video = SegmentPattern("video", MessageSegment, MessageSegment.video)
Reply = SegmentPattern("reply", MessageSegment, MessageSegment.reply)
Ark = SegmentPattern("ark", MessageSegment, MessageSegment.ark)
MarketFace = SegmentPattern("market_face", MessageSegment, MessageSegment.market_face)
Forward = SegmentPattern("forward", MessageSegment, MessageSegment.forward)


AtID = (
    UnionPattern[Union[str, MessageSegment]](
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=int,
                alias="At",
                addition_accepts=At,
                converter=lambda _, x: int(x.data["user_id"]),
            ),
            BasePattern(
                r"@(\d+)",
                mode=MatchMode.REGEX_CONVERT,
                origin=int,
                alias="@xxx",
                accepts=str,
                converter=lambda _, x: int(x[1]),
            ),
            INTEGER,
        ]
    )
    @ "at_id"
)
"""
内置类型，允许传入提醒元素(At)或者'@xxxx'式样的字符串或者数字, 返回数字
"""



from nonebot.adapters.ntchat.message import MessageSegment

from nonebot_plugin_alconna.typings import SegmentPattern

Text = str
RoomAtMsg = SegmentPattern("room_at_msg", MessageSegment, MessageSegment.room_at_msg)
Card = SegmentPattern("card", MessageSegment, MessageSegment.card)
Link = SegmentPattern("link", MessageSegment, MessageSegment.link)
Image = SegmentPattern("image", MessageSegment, MessageSegment.image)
File = SegmentPattern("file", MessageSegment, MessageSegment.file)
Video = SegmentPattern("video", MessageSegment, MessageSegment.video)
XML = SegmentPattern("xml", MessageSegment, MessageSegment.xml)



from typing import Union

from nepattern.base import URL, INTEGER
from nepattern import MatchMode, BasePattern, UnionPattern
from nonebot.adapters.onebot.v12.message import MessageSegment

from nonebot_plugin_alconna.typings import SegmentPattern

Text = str
Mention = SegmentPattern("mention", MessageSegment, MessageSegment.mention)
MentionAll = SegmentPattern("mention_all", MessageSegment, MessageSegment.mention_all)
Image = SegmentPattern("image", MessageSegment, MessageSegment.image)
Audio = SegmentPattern("audio", MessageSegment, MessageSegment.audio)
Voice = SegmentPattern("voice", MessageSegment, MessageSegment.voice)
File = SegmentPattern("file", MessageSegment, MessageSegment.file)
Video = SegmentPattern("video", MessageSegment, MessageSegment.video)
Location = SegmentPattern("location", MessageSegment, MessageSegment.location)
Reply = SegmentPattern("reply", MessageSegment, MessageSegment.reply)

ImgOrUrl = (
    UnionPattern(
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=str,
                converter=lambda _, x: x.data["url"],
                alias="img",
                addition_accepts=Image,
            ),
            URL,
        ]
    )
    @ "img_url"
)
"""
内置类型, 允许传入图片元素(Image)或者链接(URL)，返回链接
"""

MentionID = (
    UnionPattern[Union[str, MessageSegment]](
        [
            BasePattern(
                mode=MatchMode.TYPE_CONVERT,
                origin=int,
                alias="Mention",
                addition_accepts=Mention,
                converter=lambda _, x: int(x.data["user_id"]),
            ),
            BasePattern(
                r"@(\d+)",
                mode=MatchMode.REGEX_CONVERT,
                origin=int,
                alias="@xxx",
                accepts=str,
                converter=lambda _, x: int(x[1]),
            ),
            INTEGER,
        ]
    )
    @ "mention_id"
)
"""
内置类型，允许传入提醒元素(Mention)或者'@xxxx'式样的字符串或者数字, 返回数字
"""



from nonebot.adapters.ding.message import MessageSegment

from nonebot_plugin_alconna.typings import SegmentPattern

Text = str
AtAll = SegmentPattern("at", MessageSegment, MessageSegment.atAll, lambda x: "isAtAll" in x.data)
AtMobiles = SegmentPattern("at", MessageSegment, MessageSegment.atMobiles, lambda x: "atMobiles" in x.data)
AtDingtalkIds = SegmentPattern(
    "at",
    MessageSegment,
    MessageSegment.atDingtalkIds,
    lambda x: "atDingtalkIds" in x.data,
)
Image = SegmentPattern("image", MessageSegment, MessageSegment.image)
Extension = SegmentPattern("extension", MessageSegment, MessageSegment.extension)
Markdown = SegmentPattern("markdown", MessageSegment, MessageSegment.markdown)
ActionCardSingleBtn = SegmentPattern(
    "actionCard",
    MessageSegment,
    MessageSegment.actionCardSingleBtn,
    lambda x: "singleTitle" in x.data,
)
ActionCardMultiBtns = SegmentPattern(
    "actionCard",
    MessageSegment,
    MessageSegment.actionCardMultiBtns,
    lambda x: "btns" in x.data,
)
FeedCard = SegmentPattern("feedCard", MessageSegment, MessageSegment.feedCard)
Raw = SegmentPattern("raw", MessageSegment, MessageSegment.raw)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union

from tarina import lang
from nonebot.adapters import Bot, Event, Message

from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export
from ..segment import At, File, Text, AtAll, Audio, Emoji, Image, Reply, Video, Voice

if TYPE_CHECKING:
    from nonebot.adapters.qq.message import MessageSegment


class QQMessageExporter(MessageExporter["MessageSegment"]):
    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.qq

    def get_message_type(self):
        from nonebot.adapters.qq.message import Message

        return Message

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.qq.event import (
            ForumEvent,
            GuildEvent,
            ChannelEvent,
            GroupRobotEvent,
            FriendRobotEvent,
            GuildMemberEvent,
            GuildMessageEvent,
            MessageAuditEvent,
            MessageReactionEvent,
            C2CMessageCreateEvent,
            InteractionCreateEvent,
            GroupAtMessageCreateEvent,
        )

        if isinstance(event, GuildMessageEvent):
            if event.__type__.value.startswith("DIRECT"):
                return Target(
                    str(event.author.id),
                    str(event.guild_id),
                    channel=True,
                    private=True,
                    source=str(event.id),
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )  # noqa: E501
            return Target(
                str(event.channel_id),
                str(event.guild_id),
                channel=True,
                source=str(event.id),
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if isinstance(event, GuildEvent):
            return Target(
                str(event.id), channel=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None
            )
        if isinstance(event, GuildMemberEvent):
            return Target(str(event.user.id), str(event.guild_id), channel=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None)  # type: ignore # noqa: E501
        if isinstance(event, ChannelEvent):
            return Target(
                str(event.id),
                str(event.guild_id),
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if isinstance(event, MessageAuditEvent):
            return Target(
                str(event.channel_id),
                str(event.guild_id),
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if isinstance(event, MessageReactionEvent):
            return Target(
                str(event.channel_id),
                str(event.guild_id),
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if isinstance(event, ForumEvent):
            return Target(
                str(event.channel_id),
                str(event.guild_id),
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if isinstance(event, C2CMessageCreateEvent):
            return Target(
                str(event.author.id),
                private=True,
                source=str(event.id),
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
                extra={"qq.reply_seq": event._reply_seq},
            )
        if isinstance(event, GroupAtMessageCreateEvent):
            return Target(
                event.group_openid,
                source=str(event.id),
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
                extra={"qq.reply_seq": event._reply_seq},
            )
        if isinstance(event, InteractionCreateEvent):
            if event.group_openid:
                return Target(
                    event.group_openid,
                    source=str(event.id),
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
            elif event.channel_id:
                return Target(
                    event.channel_id,
                    event.guild_id or "",
                    channel=True,
                    source=str(event.id),
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
            else:
                return Target(
                    event.get_user_id(),
                    private=True,
                    source=str(event.id),
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
        if isinstance(event, FriendRobotEvent):
            return Target(event.openid, private=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        if isinstance(event, GroupRobotEvent):
            return Target(event.group_openid, platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        raise NotImplementedError

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.qq.event import (
            GuildMessageEvent,
            C2CMessageCreateEvent,
            InteractionCreateEvent,
            GroupAtMessageCreateEvent,
        )

        assert isinstance(
            event,
            (InteractionCreateEvent, GuildMessageEvent, C2CMessageCreateEvent, GroupAtMessageCreateEvent),
        )
        return str(event.id)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.style == "markdown":
            return ms.markdown(seg.text)
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.flag == "channel":
            return ms.mention_channel(seg.target)
        elif seg.flag == "user":
            return ms.mention_user(seg.target)
        else:
            raise SerializeFailed(
                lang.require("nbp-uniseg", "failed_segment").format(adapter="qq", seg=seg, target="mention")
            )

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.mention_everyone()

    @export
    async def emoji(self, seg: Emoji, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.emoji(seg.id)

    @export
    async def media(self, seg: Union[Image, Voice, Video, Audio, File], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        method = {
            "image": ms.image,
            "voice": ms.audio,
            "video": ms.video,
            "audio": ms.audio,
            "file": ms.file,
        }[name]

        file_method = {
            "image": ms.file_image,
            "voice": ms.file_audio,
            "video": ms.file_video,
            "audio": ms.file_audio,
            "file": ms.file_file,
        }[name]

        if seg.url:
            return method(seg.url)
        elif seg.raw:
            return file_method(seg.raw_bytes)
        elif seg.path:
            return file_method(Path(seg.path))
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="image", seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.reference(seg.id)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.qq.bot import Bot as QQBot
        from nonebot.adapters.qq.event import MessageEvent

        assert isinstance(bot, QQBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            assert isinstance(target, MessageEvent)
            return await bot.send(
                event=target,
                message=message,
            )

        if target.channel:
            if target.private:
                if not target.parent_id:
                    raise NotImplementedError
                dms = await bot.post_dms(recipient_id=target.id, source_guild_id=target.parent_id)
                # 私信需要使用 post_dms_messages
                # https://bot.q.qq.com/wiki/develop/api/openapi/dms/post_dms_messages.html#%E5%8F%91%E9%80%81%E7%A7%81%E4%BF%A1
                return await bot.send_to_dms(
                    guild_id=dms.guild_id,  # type: ignore
                    message=message,
                    msg_id=target.source,
                )
            return await bot.send_to_channel(
                channel_id=target.id,
                message=message,
                msg_id=target.source,
            )
        if target.private:
            res = await bot.send_to_c2c(
                openid=target.id,
                message=message,
                msg_id=target.source,
            )
        else:
            res = await bot.send_to_group(
                group_openid=target.id,
                message=message,
                msg_id=target.source,
            )
        target.extra["qq.reply_seq"] += 1
        return res

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.qq.bot import Bot as QQBot
        from nonebot.adapters.qq.event import DirectMessageCreateEvent
        from nonebot.adapters.qq.models.guild import Message as GuildMessage

        assert isinstance(bot, QQBot)
        if isinstance(mid, GuildMessage):
            if isinstance(context, Target):
                if context.private:
                    await bot.delete_dms_message(
                        guild_id=mid.guild_id,
                        message_id=mid.id,
                    )
                else:
                    await bot.delete_message(
                        channel_id=mid.channel_id,
                        message_id=mid.id,
                    )
            elif isinstance(context, DirectMessageCreateEvent):
                await bot.delete_dms_message(
                    guild_id=mid.guild_id,
                    message_id=mid.id,
                )
            else:
                await bot.delete_message(
                    channel_id=mid.channel_id,
                    message_id=mid.id,
                )
        return

    def get_reply(self, mid: Any):
        from nonebot.adapters.qq.models.guild import Message as GuildMessage

        if isinstance(mid, GuildMessage):
            return Reply(mid.id)
        raise NotImplementedError



from nepattern import type_parser
from arclet.alconna import Args, Alconna


def test_tg():
    from nonebot_plugin_alconna import Image
    from nonebot_plugin_alconna.adapters.telegram import Bold, Photo, Underline

    msg = "/com" + Bold("mand some_arg") + " " + Underline("some_arg ") + "some_arg"

    alc = Alconna("/command", Args["some_arg", Bold]["some_arg1", Underline]["some_arg2", str])

    res = alc.parse(msg)
    assert res.matched
    assert res.some_arg.type == "bold"
    assert res.some_arg1.type == "underline"
    assert isinstance(res.some_arg2, str)

    msg1 = "/command " + Bold("foo bar baz")

    alc1 = Alconna("/command", Args["foo", str]["bar", Bold]["baz", Bold])

    res1 = alc1.parse(msg1)
    assert res1.matched
    assert isinstance(res1.foo, str)
    assert res1.bar.type == "bold"
    assert res1.baz.data["text"] == "baz"

    _img = type_parser(Image)
    assert _img.validate(Photo("1")).value() == Image(id="1")



from arclet.alconna import Args, Option, Alconna


def test_fallback():
    import nonebot_plugin_alconna  # noqa: F401

    assert Alconna("test_fallback").parse("test_fallback").matched


def test_v11():
    from nonebot.adapters.onebot.v11 import Message

    import nonebot_plugin_alconna  # noqa: F401
    from nonebot_plugin_alconna.adapters.onebot11 import At, Image

    msg = Message("Hello!11") + At(123)
    img = Image(b"123")
    assert str(msg) == "Hello!11[CQ:at,qq=123]"

    alc = Alconna("Hello!11", Args["target", At])
    res = alc.parse(msg)
    assert res.matched
    assert res.target.data["qq"] == "123"
    assert not alc.parse(Message("Hello!11") + img).matched

    alc1 = Alconna(Image)
    assert alc1.parse(Message([img])).header_match.origin == img

    alc2 = Alconna([At(114514)], "chat")
    assert alc2.parse(Message(At(114514)) + "chat").matched
    assert not alc2.parse(Message(At(11454)) + "chat").matched
    assert not alc2.parse(Message(img) + "chat").matched

    alc3 = Alconna([At], "give")
    assert alc3.parse(At(114514) + "give").matched
    assert alc3.parse(At(1919810) + "give").matched
    assert not alc3.parse(img + "give").matched


def test_v12():
    from nonebot.adapters.onebot.v12 import Message

    import nonebot_plugin_alconna  # noqa: F401
    from nonebot_plugin_alconna.adapters.onebot12 import Image, Mention

    msg = Message("Hello!12") + Mention("123")
    img = Image("1.png")
    assert str(msg) == "Hello!12[mention:user_id=123]"

    alc = Alconna("Hello!12", Args["target", Mention])
    res = alc.parse(msg)
    assert res.matched
    assert res.target.data["user_id"] == "123"
    assert not alc.parse(Message("Hello!") + img).matched

    alc1 = Alconna(Image)
    assert alc1.parse(Message([img])).header_match.origin == img

    alc2 = Alconna([Mention("114514")], "chat")
    assert alc2.parse(Mention("114514") + "chat").matched
    assert not alc2.parse(Mention("11454") + "chat").matched
    assert not alc2.parse(img + "chat").matched

    msg1 = Message("Hello! --foo 123")
    assert str(msg1) == "Hello! --foo 123"

    alc3 = Alconna("Hello!", Option("--foo", Args["foo", int]))
    res1 = alc3.parse(msg1)
    assert res1.matched
    assert res1.query("foo.foo") == 123
    assert not alc3.parse("Hello!" + img).matched

    alc4 = Alconna([Mention], "give")
    assert alc4.parse(Mention("114514") + "give").matched
    assert alc4.parse(Mention("1919810") + "give").matched
    assert not alc4.parse(img + "give").matched


def test_generic():
    from nonebot.adapters.onebot.v11 import Message as Onebot11Message
    from nonebot.adapters.onebot.v12 import Message as Onebot12Message

    import nonebot_plugin_alconna  # noqa: F401
    from nonebot_plugin_alconna.uniseg import At as GenericAt
    from nonebot_plugin_alconna.adapters.onebot11 import At as Onebot11At
    from nonebot_plugin_alconna.adapters.onebot11 import Image as Onebot11Image
    from nonebot_plugin_alconna.adapters.onebot12 import Image as Onebot12Image
    from nonebot_plugin_alconna.adapters.onebot12 import Mention as Onebot12Mention

    msg11 = Onebot11Message("Hello!") + Onebot11At(123)
    msg12 = Onebot12Message("Hello!") + Onebot12Mention("123")
    img11 = Onebot11Image(b"123")
    img12 = Onebot12Image("1.png")
    assert str(msg11) == "Hello![CQ:at,qq=123]"
    assert str(msg12) == "Hello![mention:user_id=123]"

    alc = Alconna("Hello!", Args["target", GenericAt])

    assert alc.parse(msg11).matched
    assert alc.parse(msg12).matched
    assert alc.parse(msg11).target.target == "123"
    assert alc.parse(msg12).target.target == "123"
    assert not alc.parse(Onebot11Message("Hello!") + img11).matched
    assert not alc.parse(Onebot12Message("Hello!") + img12).matched



"""
Name: fleep.py
Description: File format determination library
Author: Mykyta Paliienko
License: MIT
"""

import json
from pathlib import Path

with (Path(__file__).parent / "data.json").open(encoding="utf-8") as data_file:
    data = json.load(data_file)


class Info:
    """
    Generates object with given arguments

    Args:
        types (list) -> list of file types
        extensions (list) -> list of file extensions
        mimes (list) -> list of file MIME types

    Returns:
        (<class 'fleep.Info'>) -> Class instance
    """

    def __init__(self, types: list, extensions: list, mimes: list):
        self.types = types
        self.extensions = extensions
        self.mimes = mimes

    def type_matches(self, type_: str):
        """Checks if file type matches with given type"""
        return type_ in self.types

    def extension_matches(self, extension: str):
        """Checks if file extension matches with given extension"""
        return extension in self.extensions

    def mime_matches(self, mime: str):
        """Checks if file MIME type matches with given MIME type"""
        return mime in self.mimes


def get(obj: bytes):
    """
    Determines file format and picks suitable file types, extensions and MIME types

    Args:
        obj (bytes) -> byte sequence (128 bytes are enough)

    Returns:
        (<class 'fleep.Info'>) -> Class instance
    """

    if not isinstance(obj, bytes):
        raise TypeError("object type must be bytes")

    stream = " ".join([f"{byte:02X}" for byte in obj])

    types = {}
    extensions = {}
    mimes = {}
    for element in data:
        for signature in element["signature"]:
            offset = element["offset"] * 2 + element["offset"]
            if signature == stream[offset : len(signature) + offset]:
                types[element["type"]] = len(signature)
                extensions[element["extension"]] = len(signature)
                mimes[element["mime"]] = len(signature)
    return Info(
        sorted(types, key=lambda x: types.get(x, False), reverse=True),
        sorted(extensions.keys(), key=lambda x: extensions.get(x, False), reverse=True),
        sorted(mimes.keys(), key=lambda x: mimes.get(x, False), reverse=True),
    )


def supported_types():
    """Returns a list of supported file types"""
    return sorted({x["type"] for x in data})


def supported_extensions():
    """Returns a list of supported file extensions"""
    return sorted({x["extension"] for x in data})


def supported_mimes():
    """Returns a list of supported file MIME types"""
    return sorted({x["mime"] for x in data})



from typing import Type
from typing_extensions import Annotated

from nonebot.internal.params import Depends
from nonebot.internal.adapter import Bot, Event

from .export import Target
from .message import TS, UniMessage


async def _uni_msg(bot: Bot, event: Event) -> UniMessage:
    return await UniMessage.generate(event=event, bot=bot)


def _target(bot: Bot, event: Event) -> Target:
    return UniMessage.get_target(event=event, bot=bot)


def _msg_id(bot: Bot, event: Event) -> str:
    return UniMessage.get_message_id(event=event, bot=bot)


def MessageTarget() -> Target:
    return Depends(_target, use_cache=True)


def UniversalMessage() -> UniMessage:
    return Depends(_uni_msg, use_cache=True)


def MessageId() -> str:
    return Depends(_msg_id, use_cache=True)


def UniversalSegment(t: Type[TS], index: int = 0) -> TS:
    async def _uni_seg(bot: Bot, event: Event) -> TS:
        return (await UniMessage.generate(event=event, bot=bot))[t, index]

    return Depends(_uni_seg, use_cache=True)


UniMsg = Annotated[UniMessage, UniversalMessage()]
MsgId = Annotated[str, MessageId()]
MsgTarget = Annotated[Target, MessageTarget()]



from nonebot.adapters.telegram.message import File
from nepattern import MatchMode, BasePattern, UnionPattern
from nonebot.adapters.telegram.message import Reply as _Reply
from nonebot.adapters.telegram.message import Entity, Message, UnCombinFile, MessageSegment

from nonebot_plugin_alconna.argv import MessageArgv
from nonebot_plugin_alconna.typings import SegmentPattern, TextSegmentPattern


def is_text(x: MessageSegment):
    return x.type in {
        "text",
        "bot_command",
        "bold",
        "italic",
        "underline",
        "strikethrough",
        "spoiler",
        "code",
    }


styles = {
    "record": {},
    "index": 0,
    "msg": "",
}


def builder(self: MessageArgv, data: Message):
    styles["msg"] = str(data)
    _index = 0
    for index, unit in enumerate(data):
        if not self.is_text(unit):
            self.raw_data.append(unit)
            self.ndata += 1
            continue
        if not unit.data["text"].strip():
            if not index or index == len(data) - 1:
                continue
            if not self.is_text(data[index - 1]) or not self.is_text(data[index + 1]):
                continue
        text = unit.data["text"]
        if unit.type == "text":
            self.raw_data.append(text)
            self.ndata += 1
            continue
        if self.raw_data and self.raw_data[-1].__class__ is str:
            self.raw_data[-1] = f"{self.raw_data[-1]}{text}"
        else:
            self.raw_data.append(text)
            self.ndata += 1
        start = styles["msg"].find(text, _index)
        _index = start + len(text)
        styles["record"][(start, _index)] = unit.type


def clean_style():
    styles["record"].clear()
    styles["index"] = 0


MessageArgv.custom_build(Message, is_text=is_text, builder=builder, cleanup=clean_style)  # type: ignore


def locator(x: str, t: str):
    start = styles["msg"].find(x, styles["index"])
    if start == -1:
        return False
    styles["index"] = start + len(x)
    if (maybe := styles["record"].get((start, styles["index"]))) and maybe == t:
        return True
    return any(
        scale[0] <= start <= scale[1] and scale[0] <= styles["index"] <= scale[1] and styles["record"][scale] == t
        for scale in styles["record"]
    )


Text = str
Location = SegmentPattern("location", MessageSegment, MessageSegment.location)
Venue = SegmentPattern("venue", MessageSegment, MessageSegment.venue)
Poll = SegmentPattern("poll", MessageSegment, MessageSegment.poll)
Dice = SegmentPattern("dice", MessageSegment, MessageSegment.dice)
ChatAction = SegmentPattern("chat_action", MessageSegment, MessageSegment.chat_action)

Mention = SegmentPattern("mention", Entity, Entity.mention)
Hashtag = SegmentPattern("hashtag", Entity, Entity.hashtag)
Cashtag = SegmentPattern("cashtag", Entity, Entity.cashtag)
BotCommand = TextSegmentPattern("bot_command", Entity, Entity.bot_command, locator)
Url = SegmentPattern("url", Entity, Entity.url)
Email = SegmentPattern("email", Entity, Entity.email)
PhoneNumber = SegmentPattern("phone_number", Entity, Entity.phone_number)
Bold = TextSegmentPattern("bold", Entity, Entity.bold, locator)
Italic = TextSegmentPattern("italic", Entity, Entity.italic, locator)
Underline = TextSegmentPattern("underline", Entity, Entity.underline, locator)
Strikethrough = TextSegmentPattern("strikethrough", Entity, Entity.strikethrough, locator)
Spoiler = TextSegmentPattern("spoiler", Entity, Entity.spoiler, locator)
Code = TextSegmentPattern("code", Entity, Entity.code, locator)
Pre = SegmentPattern("pre", Entity, Entity.pre)
TextLink = SegmentPattern("text_link", Entity, Entity.text_link)
TextMention = SegmentPattern("text_mention", Entity, Entity.text_mention)
CustomEmoji = SegmentPattern("custom_emoji", Entity, Entity.custom_emoji)

Image = Photo = SegmentPattern("photo", File, File.photo)
Voice = SegmentPattern("voice", File, File.voice)
Animation = SegmentPattern("animation", File, File.animation)
Audio = SegmentPattern("audio", File, File.audio)
Document = SegmentPattern("document", File, File.document)
Video = SegmentPattern("video", File, File.video)

Reply = SegmentPattern("reply", _Reply, _Reply.reply)

Sticker = SegmentPattern("sticker", UnCombinFile, UnCombinFile.sticker)
VideoNote = SegmentPattern("video_note", UnCombinFile, UnCombinFile.video_note)

Mentions = UnionPattern([Mention, TextMention, TextLink])
"""联合接收 Mention, TextMention, TextLink, 不能用于发送"""

Videos = UnionPattern([Video, Animation])
"""联合接收 Video, Animation, 不能用于发送"""

ImgOrUrl = UnionPattern(
    [
        BasePattern(
            mode=MatchMode.TYPE_CONVERT,
            origin=str,
            converter=lambda _, x: x.data["file_id"],
            alias="img",
            addition_accepts=Image,
        ),
        BasePattern(
            mode=MatchMode.TYPE_CONVERT,
            origin=str,
            converter=lambda _, x: x.data["text"],
            alias="url",
            addition_accepts=Url,
        ),
    ]
)
"""
内置类型, 允许传入图片元素(Image)或者链接(URL)，返回链接
"""



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union

from tarina import lang
from nonebot.adapters import Bot, Event, Message

from ..segment import At, File, Text, AtAll, Audio, Image, Reply, Video, Voice
from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export

if TYPE_CHECKING:
    from nonebot.adapters.onebot.v12.message import MessageSegment


class Onebot12MessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.onebot.v12.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.onebot12

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        if channel_id := getattr(event, "channel_id", None):
            guild_id = getattr(event, "guild_id", None)
            return Target(
                str(channel_id),
                str(guild_id) if guild_id else "",
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        if guild_id := getattr(event, "guild_id", None):
            return Target(
                str(guild_id), channel=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None
            )
        if group_id := getattr(event, "group_id", None):
            return Target(str(group_id), platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        if user_id := getattr(event, "user_id", None):
            return Target(str(user_id), private=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None)
        raise NotImplementedError

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.onebot.v12.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.message_id)

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.mention(seg.target)

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.mention_all()

    @export
    async def media(self, seg: Union[Image, Voice, Video, Audio, File], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        method = {
            "image": ms.image,
            "voice": ms.voice,
            "video": ms.video,
            "audio": ms.audio,
            "file": ms.file,
        }[name]
        if seg.id:
            return method(seg.id)
        elif seg.url:
            resp = await bot.upload_file(type="url", name=seg.name, url=seg.url)
            return method(resp["file_id"])
        elif seg.path:
            resp = await bot.upload_file(type="path", path=str(seg.path), name=Path(seg.path).name)
            return method(resp["file_id"])
        elif seg.raw:
            resp = await bot.upload_file(type="data", data=seg.raw_bytes, name=seg.name)
            return method(resp["file_id"])
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.reply(seg.id)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.onebot.v12.bot import Bot as OnebotBot

        assert isinstance(bot, OnebotBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)
        if target.private:
            return await bot.send_message(detail_type="private", user_id=target.id, message=message)
        elif target.channel:
            return await bot.send_message(
                detail_type="channel", channel_id=target.id, guild_id=target.parent_id, message=message
            )
        else:
            return await bot.send_message(detail_type="group", group_id=target.id, message=message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.onebot.v12.bot import Bot as OnebotBot

        assert isinstance(bot, OnebotBot)

        await bot.delete_message(message_id=mid["message_id"])
        return

    def get_reply(self, mid: Any):
        return Reply(mid["message_id"])



import asyncio

import pytest
from nonebug import App
from nonebot import get_adapter
from arclet.alconna import Args, Alconna
from nonebot.adapters.qq import Bot, Adapter, Message

from tests.fake import fake_message_event_guild


@pytest.mark.asyncio()
async def test_send(app: App):
    from nonebot_plugin_alconna import At, on_alconna

    def check(name: str):
        asyncio.create_task(bot.send(event, "check running"))
        if isinstance(name, str):
            return name
        return None

    cmd = on_alconna(Alconna([At], "test", Args["name", check]))

    @cmd.handle()
    async def _():
        await cmd.finish("test!")

    async with app.test_matcher(cmd) as ctx:
        adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter, bot_info=None)
        event = fake_message_event_guild(message=Message("<@5678> test aaaa"), id="123")
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, "check running")
        ctx.should_call_send(event, "test!")
        ctx.should_finished()



from __future__ import annotations

from typing_extensions import Self
from typing import TypeVar, Callable, Iterable

from tarina import lang
from arclet.alconna import NullMessage
from nonebot.adapters import Message, MessageSegment
from arclet.alconna.argv import Argv, set_default_argv_type

from .uniseg import Segment, UniMessage, FallbackMessage

TM = TypeVar("TM", Message, UniMessage)


def _default_builder(self: MessageArgv, data: Message | UniMessage):
    for unit in data:
        if not self.is_text(unit):
            self.raw_data.append(unit)
            self.ndata += 1
        elif res := unit.data["text"].strip():
            self.raw_data.append(res)
            self.ndata += 1


class MessageArgv(Argv[TM]):
    is_text: Callable[[MessageSegment | Segment], bool]

    @classmethod
    def custom_build(
        cls,
        target: type[TM],
        is_text: Callable[[MessageSegment | Segment], bool] = lambda x: x.is_text(),
        builder: Callable[[MessageArgv, TM], None] = _default_builder,
        cleanup: Callable[..., None] = lambda: None,
    ):
        cls._cache.setdefault(target, {}).update(
            {
                "is_text": is_text,
                "builder": builder,
                "cleanup": cleanup,
            }
        )

    def __post_init__(self):
        super().__post_init__()
        self.is_text = lambda x: x.is_text()

    @staticmethod
    def generate_token(data: list) -> int:
        return hash("".join(i.__class__.__name__ + i.__repr__() for i in data))

    def build(self, data: TM) -> Self:
        """命令分析功能, 传入字符串或消息链

        Args:
            data (TDC): 命令

        Returns:
            Self: 自身
        """
        self.reset()
        if not isinstance(data, (Message, UniMessage)):
            data = FallbackMessage(data)  # type: ignore
        cache = self.__class__._cache.get(data.__class__, {})
        if "cleanup" in cache:
            cache["cleanup"]()
        self.is_text = cache.get("is_text", self.is_text)
        self.converter = lambda x: data.__class__(x)
        self.origin = data
        cache.get("builder", _default_builder)(self, data)
        if self.ndata < 1:
            raise NullMessage(lang.require("argv", "null_message").format(target=data))
        self.bak_data = self.raw_data.copy()
        if self.message_cache:
            self.token = self.generate_token(self.raw_data)
        return self

    def addon(self, data: Iterable[str | MessageSegment]) -> Self:
        """添加命令元素

        Args:
            data (Iterable[str | MessageSegment]): 命令元素

        Returns:
            Self: 自身
        """
        for i, d in enumerate(data):
            if not d:
                continue
            if d.__class__ is str:
                text = d
            elif self.is_text(d):  # type: ignore
                text = d.data["text"]  # type: ignore
            else:
                self.raw_data.append(d)
                self.ndata += 1
                continue
            if not text.strip("\xa0").strip():  # type: ignore
                continue
            if i > 0 and isinstance(self.raw_data[-1], str):
                self.raw_data[-1] += f"{self.separators[0]}{text}"
            else:
                self.raw_data.append(text)
                self.ndata += 1
        self.bak_data = self.raw_data.copy()
        if self.message_cache:
            self.token = self.generate_token(self.raw_data)
        return self


set_default_argv_type(MessageArgv)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union, cast

from tarina import lang
from nonebot.drivers import Request
from nonebot.adapters import Bot, Event, Message

from ..segment import At, Text, Image, Reply, Video
from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export

if TYPE_CHECKING:
    from nonebot.adapters.dodo.message import MessageSegment


class DoDoMessageExporter(MessageExporter["MessageSegment"]):
    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.dodo

    def get_message_type(self):
        from nonebot.adapters.dodo.message import Message

        return Message

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.dodo.event import Event as DoDoEvent

        assert isinstance(event, DoDoEvent)
        channel_id = getattr(event, "channel_id", None)
        island_id = getattr(event, "island_source_id", None)
        if channel_id:
            return Target(
                channel_id,
                island_id or "",
                True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        else:
            return Target(
                event.user_id,
                island_id or "",
                True,
                True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.dodo.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return event.message_id

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        if seg.flag == "user":
            return ms.at_user(seg.target)
        elif seg.flag == "channel":
            return ms.channel_link(seg.target)
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="at", seg=seg))

    @export
    async def image(self, seg: Image, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if TYPE_CHECKING:
            from nonebot.adapters.dodo.bot import Bot as DoDoBot

            assert isinstance(bot, DoDoBot)
        if seg.raw:
            data = seg.raw_bytes
        elif seg.path:
            data = Path(seg.path)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            data = cast(bytes, resp.content)
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="image", seg=seg))
        res = await bot.set_resouce_picture_upload(file=data)

        return ms.picture(res.url, res.width, res.height)

    @export
    async def video(self, seg: Video, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.url:
            return ms.video(seg.url)
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="video", seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.reference(seg.id)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.dodo.bot import Bot as DodoBot

        assert isinstance(bot, DodoBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        if target.private:
            return await bot.send_to_personal(target.parent_id, target.id, message)
        else:
            return await bot.send_to_channel(target.id, message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.dodo.bot import Bot as DodoBot

        assert isinstance(bot, DodoBot)
        if isinstance(context, Target) and not context.private:
            return await bot.set_channel_message_withdraw(message_id=mid)
        elif hasattr(context, "channel_id"):
            return await bot.set_channel_message_withdraw(message_id=mid)

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.dodo.bot import Bot as DodoBot

        assert isinstance(bot, DodoBot)
        if TYPE_CHECKING:
            assert isinstance(new, self.get_message_type())
        if isinstance(context, Target) and not context.private:
            return await bot.set_channel_message_edit(message_id=mid, message_body=new.to_message_body()[0])
        elif hasattr(context, "channel_id"):
            return await bot.set_channel_message_edit(message_id=mid, message_body=new.to_message_body()[0])

    def get_reply(self, mid: Any):
        return Reply(mid)



from typing import Any

import pytest
from nonebug import App
from nonebot import get_adapter
from arclet.alconna import Args, Alconna
from nonebot.adapters.onebot.v11 import Bot, Adapter, Message

from tests.fake import fake_group_message_event_v11


@pytest.mark.asyncio()
async def test_extension(app: App):
    from nonebot.adapters.onebot.v11 import MessageEvent

    from nonebot_plugin_alconna import Extension, Interface, on_alconna

    class DemoExtension(Extension):
        @property
        def priority(self) -> int:
            return 1

        @property
        def id(self) -> str:
            return "demo"

        async def permission_check(self, bot, event, command):
            if event.get_user_id() != "123":
                await bot.send(event, "权限不足！")
                return False
            return True

        def before_catch(self, name: str, annotation: Any, default: Any) -> bool:
            if annotation is str:
                return True
            return False

        async def catch(self, interface: Interface[MessageEvent]):
            if interface.annotation is str:
                return {
                    "hello": "Hello!",
                    "world": "World!",
                }.get(interface.name, interface.name)

    add = on_alconna(Alconna("add", Args["a", float]["b", float]), extensions=[DemoExtension], comp_config={})

    @add.handle()
    async def h(a: float, b: float, hello: str, world: str, test: str):
        """加法测试"""
        await add.send(f"{a} + {b} = {a + b}\n{hello} {world} {test}!")

    async with app.test_matcher(add) as ctx:  # type: ignore
        adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter)
        event = fake_group_message_event_v11(message=Message("add 1.3 2.4"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, "1.3 + 2.4 = 3.7\nHello! World! test!")

        event = fake_group_message_event_v11(message=Message("add 1.3 2.4"), user_id=456)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, "权限不足！")



import pytest
from nonebug import App
from nonebot import get_adapter
from arclet.alconna import Args, Alconna
from nonebot.adapters.satori import Bot, Adapter, Message, MessageSegment

from tests.fake import fake_message_event_satori


def test_satori():
    from nonebot.adapters.satori import Message

    from nonebot_plugin_alconna.adapters.satori import Bold, Underline

    msg = Message("/com<b>mand some_arg</b> <u>some_arg</u> some_arg")

    alc = Alconna("/command", Args["some_arg", Bold]["some_arg1", Underline]["some_arg2", str])

    res = alc.parse(msg)
    assert res.matched
    assert res.some_arg.type == "bold"
    assert res.some_arg1.type == "underline"
    assert isinstance(res.some_arg2, str)

    msg1 = "/command " + Bold("foo bar baz")

    alc1 = Alconna("/command", Args["foo", str]["bar", Bold]["baz", Bold])

    res1 = alc1.parse(msg1)
    assert res1.matched
    assert isinstance(res1.foo, str)
    assert res1.bar.type == "bold"
    assert res1.baz.data["text"] == "baz"


@pytest.mark.asyncio()
async def test_send(app: App):
    from nonebot_plugin_alconna import Text, Image, on_alconna

    test_cmd = on_alconna(Alconna("test", Args["img", Image]))

    @test_cmd.handle()
    async def tt_h(img: Image):
        await test_cmd.send(Text("ok\n") + img)

    async with app.test_matcher(test_cmd) as ctx:
        adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter, platform="satori", info=None)
        msg = "test" + MessageSegment.image(raw={"data": b"123", "mime": "image/png"})
        event = fake_message_event_satori(message=msg, id=123)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, Message('ok\n<img src="data:image/png;base64,MTIz" />'))



import asyncio
from io import BytesIO
from pathlib import Path
from copy import deepcopy
from types import FunctionType
from dataclasses import dataclass
from typing_extensions import Self, SupportsIndex
from typing import TYPE_CHECKING, Any, List, Type, Tuple, Union, Literal, TypeVar, Iterable, Optional, overload

from tarina import lang
from nonebot.internal.adapter import Bot, Event, Message
from nonebot.internal.matcher import current_bot, current_event

from .tools import get_bot
from .adapters import MAPPING
from .fallback import FallbackMessage
from .template import UniMessageTemplate
from .export import Target, MessageExporter, SerializeFailed
from .segment import (
    At,
    Card,
    File,
    Text,
    AtAll,
    Audio,
    Emoji,
    Image,
    Other,
    Reply,
    Video,
    Voice,
    Segment,
    reply,
    segments,
    reply_handle,
)

T = TypeVar("T")
TS = TypeVar("TS", bound=Segment)
TS1 = TypeVar("TS1", bound=Segment)


class _method:
    def __init__(self, func: FunctionType):
        self.__func__ = func

    def __get__(self, instance, owner):
        if instance is None:
            return self.__func__.__get__(owner, owner)
        return self.__func__.__get__(instance, owner)


class UniMessage(List[TS]):
    """通用消息序列

    参数:
        message: 消息内容
    """

    if TYPE_CHECKING:

        @classmethod
        def text(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]], text: str  # type: ignore
        ) -> "UniMessage[Union[TS1, Text]]":
            """创建纯文本消息

            参数:
                text: 文本内容

            返回:
                构建的消息
            """
            ...

        @classmethod
        def at(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]], user_id: str  # type: ignore
        ) -> "UniMessage[Union[TS1, At]]":
            """创建 @用户 消息

            参数:
                user_id: 要 @ 的用户 ID

            返回:
                构建的消息
            """
            ...

        @classmethod
        def at_role(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]], role_id: str  # type: ignore
        ) -> "UniMessage[Union[TS1, At]]":
            """创建 @角色组 消息

            参数:
                role_id: 要 @ 的角色 ID

            返回:
                构建的消息
            """
            ...

        @classmethod
        def at_channel(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]], channel_id: str  # type: ignore
        ) -> "UniMessage[Union[TS1, At]]":
            """创建 #频道 消息

            参数:
                channel_id: 要 @ 的频道 ID

            返回:
                构建的消息
            """
            ...

        @classmethod
        def at_all(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            online: bool = False,
        ) -> "UniMessage[Union[TS1, AtAll]]":
            """创建 @全体成员 消息

            参数:
                online: 是否只 @ 在线成员

            返回:
                构建的消息
            """
            ...

        @classmethod
        def emoji(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: str,
            name: Optional[str] = None,
        ) -> "UniMessage[Union[TS1, Emoji]]":
            """创建 emoji 消息

            参数:
                id: emoji ID
                name: emoji 名称

            返回:
                构建的消息
            """
            ...

        @classmethod
        def image(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "image.png",
        ) -> "UniMessage[Union[TS1, Image]]":
            """创建图片消息

            参数:
                id: 图片 ID
                url: 图片链接
                path: 图片路径
                raw: 图片原始数据
                mimetype: 图片 MIME 类型
                name: 图片名称
            返回:
                构建的消息
            """
            ...

        @classmethod
        def video(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "video.mp4",
        ) -> "UniMessage[Union[TS1, Video]]":
            """创建视频消息

            参数:
                id: 视频 ID
                url: 视频链接
                path: 视频路径
                raw: 视频原始数据
                mimetype: 视频 MIME 类型
                name: 视频名称
            返回:
                构建的消息
            """
            ...

        @classmethod
        def voice(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "voice.wav",
        ) -> "UniMessage[Union[TS1, Voice]]":
            """创建语音消息

            参数:
                id: 语音 ID
                url: 语音链接
                path: 语音路径
                raw: 语音原始数据
                mimetype: 语音 MIME 类型
                name: 语音名称
            返回:
                构建的消息
            """
            ...

        @classmethod
        def audio(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "audio.mp3",
        ) -> "UniMessage[Union[TS1, Audio]]":
            """创建音频消息

            参数:
                id: 音频 ID
                url: 音频链接
                path: 音频路径
                raw: 音频原始数据
                mimetype: 音频 MIME 类型
                name: 音频名称
            返回:
                构建的消息
            """
            ...

        @classmethod
        def file(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "file.bin",
        ) -> "UniMessage[Union[TS1, File]]":
            """创建文件消息

            参数:
                id: 文件 ID
                url: 文件链接
                path: 文件路径
                raw: 文件原始数据
                mimetype: 文件 MIME 类型
                name: 文件名称
            返回:
                构建的消息
            """
            ...

        @classmethod
        def reply(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]], id: str  # type: ignore
        ) -> "UniMessage[Union[TS1, Reply]]":
            """创建回复消息

            参数:
                id: 回复消息 ID

            返回:
                构建的消息
            """
            ...

        @classmethod
        def card(
            cls_or_self: Union["UniMessage[TS1]", Type["UniMessage[TS1]"]],  # type: ignore
            flag: Literal["xml", "json"],
            content: str,
        ) -> "UniMessage[Union[TS1, Card]]":
            """创建卡片消息

            参数:
                flag: 卡片类型
                content: 卡片内容

            返回:
                构建的消息
            """
            ...

    else:

        @_method
        def text(cls_or_self, text: str) -> "UniMessage[Union[TS1, Text]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Text(text))
                return cls_or_self
            return UniMessage(Text(text))

        @_method
        def at(cls_or_self, user_id: str) -> "UniMessage[Union[TS1, At]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(At("user", user_id))
                return cls_or_self
            return UniMessage(At("user", user_id))

        @_method
        def at_role(cls_or_self, role_id: str) -> "UniMessage[Union[TS1, At]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(At("role", role_id))
                return cls_or_self
            return UniMessage(At("role", role_id))

        @_method
        def at_channel(cls_or_self, channel_id: str) -> "UniMessage[Union[TS1, At]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(At("channel", channel_id))
                return cls_or_self
            return UniMessage(At("channel", channel_id))

        @_method
        def at_all(cls_or_self, online: bool = False) -> "UniMessage[Union[TS1, AtAll]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(AtAll(online))
                return cls_or_self
            return UniMessage(AtAll(online))

        @_method
        def emoji(cls_or_self, id: str, name: Optional[str] = None) -> "UniMessage[Union[TS1, Emoji]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Emoji(id, name))
                return cls_or_self
            return UniMessage(Emoji(id, name))

        @_method
        def image(
            cls_or_self,
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "image.png",
        ) -> "UniMessage[Union[TS1, Image]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Image(id, url, path, raw, mimetype, name))
                return cls_or_self
            return UniMessage(Image(id, url, path, raw, mimetype, name))

        @_method
        def video(
            cls_or_self,
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "video.mp4",
        ) -> "UniMessage[Union[TS1, Video]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Video(id, url, path, raw, mimetype, name))
                return cls_or_self
            return UniMessage(Video(id, url, path, raw, mimetype, name))

        @_method
        def voice(
            cls_or_self,
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "voice.wav",
        ) -> "UniMessage[Union[TS1, Voice]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Voice(id, url, path, raw, mimetype, name))
                return cls_or_self
            return UniMessage(Voice(id, url, path, raw, mimetype, name))

        @_method
        def audio(
            cls_or_self,
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "audio.mp3",
        ) -> "UniMessage[Union[TS1, Audio]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Audio(id, url, path, raw, mimetype, name))
                return cls_or_self
            return UniMessage(Audio(id, url, path, raw, mimetype, name))

        @_method
        def file(
            cls_or_self,
            id: Optional[str] = None,
            url: Optional[str] = None,
            path: Optional[Union[str, Path]] = None,
            raw: Optional[Union[bytes, BytesIO]] = None,
            mimetype: Optional[str] = None,
            name: str = "file.bin",
        ) -> "UniMessage[Union[TS1, File]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(File(id, url, path, raw, mimetype, name))
                return cls_or_self
            return UniMessage(File(id, url, path, raw, mimetype, name))

        @_method
        def reply(cls_or_self, id: str) -> "UniMessage[Union[TS1, Reply]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Reply(id))
                return cls_or_self
            return UniMessage(Reply(id))

        @_method
        def card(cls_or_self, flag: Literal["xml", "json"], content: str) -> "UniMessage[Union[TS1, Card]]":
            if isinstance(cls_or_self, UniMessage):
                cls_or_self.append(Card(flag, content))
                return cls_or_self
            return UniMessage(Card(flag, content))

    def __init__(
        self: "UniMessage[Segment]",
        message: Union[Iterable[Union[str, TS]], str, TS, None] = None,
    ):
        super().__init__()
        if isinstance(message, str):
            self.__iadd__(Text(message))
        elif isinstance(message, Iterable):
            for i in message:
                self.__iadd__(Text(i) if isinstance(i, str) else i)
        elif isinstance(message, Segment):
            self.__iadd__(message)

    def __str__(self) -> str:
        return "".join(str(seg) for seg in self)

    def __repr__(self) -> str:
        return "[" + ", ".join(repr(seg) for seg in self) + "]"

    @classmethod
    def template(cls, format_string: Union[str, "UniMessage"]) -> UniMessageTemplate:
        """创建消息模板。

        用法和 `str.format` 大致相同，支持以 `UniMessage` 对象作为消息模板并输出消息对象。
        并且提供了拓展的格式化控制符，可以通过 `Segment` 的实例化方法创建消息。

        参数:
            format_string: 格式化模板

        返回:
            消息格式化器
        """
        return UniMessageTemplate(format_string, cls)

    @overload
    def __add__(self, other: str) -> "UniMessage[Union[TS, Text]]": ...

    @overload
    def __add__(self, other: Union[TS, Iterable[TS]]) -> "UniMessage[TS]": ...

    @overload
    def __add__(self, other: Union[TS1, Iterable[TS1]]) -> "UniMessage[Union[TS, TS1]]": ...

    def __add__(self, other: Union[str, TS, TS1, Iterable[Union[TS, TS1]]]) -> "UniMessage":
        result: UniMessage = self.copy()
        if isinstance(other, str):
            if result and isinstance(text := result[-1], Text):
                text.text += other
            else:
                result.append(Text(other))
        elif isinstance(other, Segment):
            if result and isinstance(result[-1], Text) and isinstance(other, Text):
                result[-1] = Text(result[-1].text + other.text)
            else:
                result.append(other)
        elif isinstance(other, Iterable):
            for seg in other:
                result += seg
        else:
            raise TypeError(f"Unsupported type {type(other)!r}")
        return result

    @overload
    def __radd__(self, other: str) -> "UniMessage[Union[Text, TS]]": ...

    @overload
    def __radd__(self, other: Union[TS, Iterable[TS]]) -> "UniMessage[TS]": ...

    @overload
    def __radd__(self, other: Union[TS1, Iterable[TS1]]) -> "UniMessage[Union[TS1, TS]]": ...

    def __radd__(self, other: Union[str, TS1, Iterable[TS1]]) -> "UniMessage":
        result = UniMessage(other)
        return result + self

    def __iadd__(self, other: Union[str, TS, Iterable[TS]]) -> Self:
        if isinstance(other, str):
            if self and isinstance(text := self[-1], Text):
                text.text += other
            else:
                self.append(Text(other))  # type: ignore
        elif isinstance(other, Segment):
            if self and (isinstance(text := self[-1], Text) and isinstance(other, Text)):
                text.text += other.text
            else:
                self.append(other)
        elif isinstance(other, Iterable):
            for seg in other:
                self.__iadd__(seg)
        else:
            raise TypeError(f"Unsupported type {type(other)!r}")
        return self

    @overload
    def __getitem__(self, args: Type[TS1]) -> "UniMessage[TS1]":
        """获取仅包含指定消息段类型的消息

        参数:
            args: 消息段类型

        返回:
            所有类型为 `args` 的消息段
        """

    @overload
    def __getitem__(self, args: Tuple[Type[TS1], int]) -> TS1:
        """索引指定类型的消息段

        参数:
            args: 消息段类型和索引

        返回:
            类型为 `args[0]` 的消息段第 `args[1]` 个
        """

    @overload
    def __getitem__(self, args: Tuple[Type[TS1], slice]) -> "UniMessage[TS1]":
        """切片指定类型的消息段

        参数:
            args: 消息段类型和切片

        返回:
            类型为 `args[0]` 的消息段切片 `args[1]`
        """

    @overload
    def __getitem__(self, args: int) -> TS:
        """索引消息段

        参数:
            args: 索引

        返回:
            第 `args` 个消息段
        """

    @overload
    def __getitem__(self, args: slice) -> "UniMessage[TS]":
        """切片消息段

        参数:
            args: 切片

        返回:
            消息切片 `args`
        """

    def __getitem__(
        self,
        args: Union[
            Type[TS1],
            Tuple[Type[TS1], int],
            Tuple[Type[TS1], slice],
            int,
            slice,
        ],
    ) -> Union[TS, TS1, "UniMessage[TS]", "UniMessage[TS1]"]:
        arg1, arg2 = args if isinstance(args, tuple) else (args, None)
        if isinstance(arg1, int) and arg2 is None:
            return super().__getitem__(arg1)
        if isinstance(arg1, slice) and arg2 is None:
            return UniMessage(super().__getitem__(arg1))
        if TYPE_CHECKING:
            assert not isinstance(arg1, (slice, int))
        if issubclass(arg1, Segment) and arg2 is None:
            return UniMessage(seg for seg in self if isinstance(seg, arg1))
        if issubclass(arg1, Segment) and isinstance(arg2, int):
            return [seg for seg in self if isinstance(seg, arg1)][arg2]
        if issubclass(arg1, Segment) and isinstance(arg2, slice):
            return UniMessage([seg for seg in self if isinstance(seg, arg1)][arg2])
        raise ValueError("Incorrect arguments to slice")  # pragma: no cover

    def __contains__(self, value: Union[str, Segment, Type[Segment]]) -> bool:
        """检查消息段是否存在

        参数:
            value: 消息段或消息段类型
        返回:
            消息内是否存在给定消息段或给定类型的消息段
        """
        if isinstance(value, type):
            return bool(next((seg for seg in self if isinstance(seg, value)), None))
        if isinstance(value, str):
            value = Text(value)
        return super().__contains__(value)

    def has(self, value: Union[str, Segment, Type[Segment]]) -> bool:
        """与 {ref}``__contains__` <nonebot.adapters.Message.__contains__>` 相同"""
        return value in self

    def index(self, value: Union[str, Segment, Type[Segment]], *args: SupportsIndex) -> int:
        """索引消息段

        参数:
            value: 消息段或者消息段类型
            arg: start 与 end

        返回:
            索引 index

        异常:
            ValueError: 消息段不存在
        """
        if isinstance(value, type):
            first_segment = next((seg for seg in self if isinstance(seg, value)), None)
            if first_segment is None:
                raise ValueError(f"Segment with type {value!r} is not in message")
            return super().index(first_segment, *args)
        if isinstance(value, str):
            value = Text(value)
        return super().index(value, *args)  # type: ignore

    def get(self, type_: Type[TS], count: Optional[int] = None) -> "UniMessage[TS]":
        """获取指定类型的消息段

        参数:
            type_: 消息段类型
            count: 获取个数

        返回:
            构建的新消息
        """
        if count is None:
            return self[type_]

        iterator, filtered = (seg for seg in self if isinstance(seg, type_)), UniMessage()
        for _ in range(count):
            seg = next(iterator, None)
            if seg is None:
                break
            filtered.append(seg)
        return filtered

    def count(self, value: Union[Type[Segment], str, Segment]) -> int:
        """计算指定消息段的个数

        参数:
            value: 消息段或消息段类型

        返回:
            个数
        """
        if isinstance(value, str):
            value = Text(value)
        return (
            len(self[value])  # type: ignore
            if isinstance(value, type)
            else super().count(value)  # type: ignore
        )

    def only(self, value: Union[Type[Segment], str, Segment]) -> bool:
        """检查消息中是否仅包含指定消息段

        参数:
            value: 指定消息段或消息段类型

        返回:
            是否仅包含指定消息段
        """
        if isinstance(value, type):
            return all(isinstance(seg, value) for seg in self)
        if isinstance(value, str):
            value = Text(value)
        return all(seg == value for seg in self)

    def join(self, iterable: Iterable[Union[TS1, "UniMessage[TS1]"]]) -> "UniMessage[Union[TS, TS1]]":
        """将多个消息连接并将自身作为分割

        参数:
            iterable: 要连接的消息

        返回:
            连接后的消息
        """
        ret = UniMessage()
        for index, msg in enumerate(iterable):
            if index != 0:
                ret.extend(self)
            if isinstance(msg, Segment):
                ret.append(msg)
            else:
                ret.extend(msg.copy())
        return ret

    def copy(self) -> "UniMessage[TS]":
        """深拷贝消息"""
        return deepcopy(self)

    def include(self, *types: Type[Segment]) -> "UniMessage[TS]":
        """过滤消息

        参数:
            types: 包含的消息段类型

        返回:
            新构造的消息
        """
        return UniMessage(seg for seg in self if seg.__class__ in types)

    def exclude(self, *types: Type[Segment]) -> "UniMessage[TS]":
        """过滤消息

        参数:
            types: 不包含的消息段类型

        返回:
            新构造的消息
        """
        return UniMessage(seg for seg in self if seg.__class__ not in types)

    def extract_plain_text(self) -> str:
        """提取消息内纯文本消息"""

        return "".join(seg.text for seg in self if isinstance(seg, Text))

    @staticmethod
    async def generate(*, message: Optional[Message] = None, event: Optional[Event] = None, bot: Optional[Bot] = None):
        if not message:
            if not event:
                try:
                    event = current_event.get()
                except LookupError as e:
                    raise SerializeFailed(lang.require("nbp-uniseg", "event_missing")) from e
            try:
                message = event.get_message()
            except Exception:
                return UniMessage()
        result = UniMessage()
        msg_copy = message.copy()
        if (event and bot) and (_reply := await reply_handle(event, bot)):
            result.append(_reply)
        elif (res := reply.validate(message[0])).success:
            res._value.origin = message[0]
            result.append(res.value)
            msg_copy.pop(0)
        for seg in msg_copy:
            for pat in segments:
                if (res := pat.validate(seg)).success:
                    res._value.origin = seg
                    result.append(res._value)
                    break
            else:
                result.append(Other(seg))
        return result

    @staticmethod
    def get_message_id(event: Optional[Event] = None, bot: Optional[Bot] = None, adapter: Optional[str] = None) -> str:
        if not event:
            try:
                event = current_event.get()
            except LookupError as e:
                raise SerializeFailed(lang.require("nbp-uniseg", "event_missing")) from e
        if not adapter:
            if not bot:
                try:
                    bot = current_bot.get()
                except LookupError as e:
                    raise SerializeFailed(lang.require("nbp-uniseg", "bot_missing")) from e
            _adapter = bot.adapter
            adapter = _adapter.get_name()
        if fn := MAPPING.get(adapter):
            return fn.get_message_id(event)
        raise SerializeFailed(lang.require("nbp-uniseg", "unsupported").format(adapter=adapter))

    @staticmethod
    def get_target(event: Optional[Event] = None, bot: Optional[Bot] = None, adapter: Optional[str] = None) -> Target:
        if not event:
            try:
                event = current_event.get()
            except LookupError as e:
                raise SerializeFailed(lang.require("nbp-uniseg", "event_missing")) from e
        if not adapter:
            if not bot:
                try:
                    bot = current_bot.get()
                except LookupError as e:
                    raise SerializeFailed(lang.require("nbp-uniseg", "bot_missing")) from e
            _adapter = bot.adapter
            adapter = _adapter.get_name()
        if fn := MAPPING.get(adapter):
            return fn.get_target(event, bot)
        raise SerializeFailed(lang.require("nbp-uniseg", "unsupported").format(adapter=adapter))

    async def export(self, bot: Optional[Bot] = None, fallback: bool = True) -> Message:
        if not bot:
            try:
                bot = current_bot.get()
            except LookupError as e:
                raise SerializeFailed(lang.require("nbp-uniseg", "bot_missing")) from e
        adapter = bot.adapter
        adapter_name = adapter.get_name()
        try:
            if fn := MAPPING.get(adapter_name):
                return await fn.export(self, bot, fallback)
            raise SerializeFailed(lang.require("nbp-uniseg", "unsupported").format(adapter=adapter_name))
        except SerializeFailed:
            if fallback:
                return FallbackMessage(str(self))
            raise

    async def send(
        self,
        target: Union[Event, Target, None] = None,
        bot: Optional[Bot] = None,
        fallback: bool = True,
        at_sender: Union[str, bool] = False,
        reply_to: Union[str, bool, Reply, None] = False,
    ) -> "Receipt":
        if not target:
            try:
                target = current_event.get()
            except LookupError as e:
                raise SerializeFailed(lang.require("nbp-uniseg", "event_missing")) from e
        if not bot:
            try:
                bot = current_bot.get()
            except LookupError as e:
                if not isinstance(target, Target) or not target.platform:
                    raise SerializeFailed(lang.require("nbp-uniseg", "bot_missing")) from e
                if target.self_id:
                    bot = get_bot(bot_id=target.self_id)
                else:
                    bot = get_bot(adapter=target.platform, rand=True)
        if at_sender:
            if isinstance(at_sender, str):
                self.insert(0, At("user", at_sender))  # type: ignore
            elif isinstance(target, Event):
                self.insert(0, At("user", target.get_user_id()))  # type: ignore
            else:
                raise TypeError("at_sender must be str when target is not Event")
        if reply_to:
            if isinstance(reply_to, Reply):
                self.insert(0, reply_to)  # type: ignore
            else:
                if isinstance(reply_to, bool):
                    if isinstance(target, Event):
                        reply_to = self.get_message_id(target, bot)
                    else:
                        raise TypeError("reply_to must be str when target is not Event")
                self.insert(0, Reply(reply_to))  # type: ignore
        msg = await self.export(bot, fallback)
        adapter = bot.adapter
        adapter_name = adapter.get_name()
        if not (fn := MAPPING.get(adapter_name)):
            raise SerializeFailed(lang.require("nbp-uniseg", "unsupported").format(adapter=adapter_name))
        res = await fn.send_to(target, bot, msg)
        return Receipt(bot, target, fn, res if isinstance(res, list) else [res])


@dataclass
class Receipt:
    bot: Bot
    context: Union[Event, Target]
    exporter: MessageExporter
    msg_ids: List[Any]

    def get_reply(self, index: int = -1) -> Union[Reply, None]:
        if not self.msg_ids:
            return
        try:
            msg_id = self.msg_ids[index]
        except IndexError:
            msg_id = self.msg_ids[0]
        if not msg_id:
            return
        try:
            return self.exporter.get_reply(msg_id)
        except NotImplementedError:
            return

    async def recall(self, delay: float = 0, index: int = -1):
        if not self.msg_ids:
            return self
        if delay > 1e-4:
            await asyncio.sleep(delay)
        try:
            msg_id = self.msg_ids[index]
        except IndexError:
            msg_id = self.msg_ids[0]
        if not msg_id:
            return self
        try:
            await self.exporter.recall(msg_id, self.bot, self.context)
            self.msg_ids.remove(msg_id)
            return self
        except NotImplementedError:
            return self

    async def edit(
        self,
        message: Union[UniMessage, str, Iterable[Union[str, Segment]], Segment],
        delay: float = 0,
        index: int = -1,
    ):
        if not self.msg_ids:
            return self
        if delay > 1e-4:
            await asyncio.sleep(delay)
        message = UniMessage(message)
        msg = await self.exporter.export(message, self.bot, True)
        try:
            msg_id = self.msg_ids[index]
        except IndexError:
            msg_id = self.msg_ids[0]
        if not msg_id:
            return self
        try:
            res = await self.exporter.edit(msg, msg_id, self.bot, self.context)
            if res:
                if isinstance(res, list):
                    self.msg_ids.remove(msg_id)
                    self.msg_ids.extend(res)
                else:
                    self.msg_ids[index] = res
            return self
        except NotImplementedError:
            return self

    async def send(
        self,
        message: Union[UniMessage, str, Iterable[Union[str, Segment]], Segment],
        fallback: bool = True,
        at_sender: Union[str, bool] = False,
        reply_to: Union[str, bool, Reply, None] = False,
        delay: float = 0,
    ):
        if delay > 1e-4:
            await asyncio.sleep(delay)
        message = UniMessage(message)
        if at_sender:
            if isinstance(at_sender, str):
                message.insert(0, At("user", at_sender))  # type: ignore
            elif isinstance(self.context, Event):
                message.insert(0, At("user", self.context.get_user_id()))  # type: ignore
            else:
                raise TypeError("at_sender must be str when target is not Event")
        if reply_to:
            if isinstance(reply_to, Reply):
                message.insert(0, reply_to)  # type: ignore
            else:
                if isinstance(reply_to, bool):
                    if isinstance(self.context, Event):
                        reply_to = self.exporter.get_message_id(self.context)
                    else:
                        raise TypeError("reply_to must be str when target is not Event")
                self.insert(0, Reply(reply_to))  # type: ignore
        msg = await self.exporter.export(message, self.bot, fallback)
        res = await self.exporter.send_to(self.context, self.bot, msg)
        self.msg_ids.extend(res if isinstance(res, list) else [res])
        return self

    async def reply(
        self,
        message: Union[UniMessage, str, Iterable[Union[str, Segment]], Segment],
        fallback: bool = True,
        at_sender: Union[str, bool] = False,
        index: int = -1,
        delay: float = 0,
    ):
        return await self.send(message, fallback, at_sender, self.get_reply(index), delay)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union

from tarina import lang
from nonebot.internal.driver import Request
from nonebot.adapters import Bot, Event, Message

from ..segment import At, File, Text, AtAll, Audio, Image, Reply, Video, Voice
from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export

if TYPE_CHECKING:
    from nonebot.adapters.feishu.message import MessageSegment


class FeishuMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.feishu.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.feishu

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.feishu.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.message_id)

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.feishu.event import GroupMessageEvent, PrivateMessageEvent

        if isinstance(event, GroupMessageEvent):
            return Target(
                event.event.message.chat_id, platform=self.get_adapter(), self_id=bot.self_id if bot else None
            )
        elif isinstance(event, PrivateMessageEvent):
            return Target(
                event.get_user_id(),
                private=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        raise NotImplementedError

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.at(seg.target)

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.at("all")

    @export
    async def image(self, seg: Image, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.id:
            return ms.image(seg.id)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            image = resp.content
        elif seg.path:
            image = Path(seg.path).read_bytes()
        elif seg.raw:
            image = seg.raw_bytes
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="image", seg=seg))
        data = {"image_type": "message"}
        files = {"image": ("file", image)}
        params = {"method": "POST", "data": data, "files": files}
        result = await bot.call_api("im/v1/images", **params)
        file_key = result["data"]["image_key"]
        return ms.image(file_key)

    @export
    async def audio(self, seg: Union[Voice, Audio], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        if seg.id:
            return ms.audio(seg.id, seg.duration)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            raw = resp.content
        elif seg.path:
            raw = Path(seg.path).read_bytes()
        elif seg.raw:
            raw = seg.raw_bytes
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))
        data = {"file_type": "stream", "file_name": seg.name}
        files = {"file": ("file", raw)}
        params = {"method": "POST", "data": data, "files": files}
        result = await bot.call_api("im/v1/files", **params)
        file_key = result["data"]["file_key"]
        return ms.audio(file_key, seg.duration)

    @export
    async def file(self, seg: File, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.id:
            return ms.file(seg.id, seg.name)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            raw = resp.content
        elif seg.path:
            raw = Path(seg.path).read_bytes()
        elif seg.raw:
            raw = seg.raw_bytes
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="file", seg=seg))
        data = {"file_type": "stream", "file_name": seg.name}
        files = {"file": ("file", raw)}
        params = {"method": "POST", "data": data, "files": files}
        result = await bot.call_api("im/v1/files", **params)
        file_key = result["data"]["file_key"]
        return ms.file(file_key, seg.name)

    @export
    async def video(self, seg: Video, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.id:
            return ms.sticker(seg.id)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            raw = resp.content
        elif seg.path:
            raw = Path(seg.path).read_bytes()
        elif seg.raw:
            raw = seg.raw_bytes
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="video", seg=seg))
        data = {"file_type": "stream", "file_name": seg.name}
        files = {"file": ("file", raw)}
        params = {"method": "POST", "data": data, "files": files}
        result = await bot.call_api("im/v1/files", **params)
        file_key = result["data"]["file_key"]
        return ms.sticker(file_key)

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms("$reply", {"message_id": seg.id})  # type: ignore

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.feishu.bot import Bot as FeishuBot

        assert isinstance(bot, FeishuBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        if target.private:
            receive_id, receive_id_type = target.id, "open_id"
        else:
            receive_id, receive_id_type = target.id, "chat_id"
        if message.has("$reply"):
            reply = message["$reply", 0]
            message = message.exclude("$reply")
            msg_type, content = message.serialize()
            return await bot.reply_msg(reply.data["message_id"], content, msg_type)
        msg_type, content = message.serialize()
        return await bot.send_msg(receive_id_type, receive_id, content, msg_type)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.feishu.bot import Bot as FeishuBot

        assert isinstance(bot, FeishuBot)

        params = {"method": "DELETE"}
        return await bot.call_api(f"im/v1/messages/{mid['message_id']}", **params)

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.feishu.bot import Bot as FeishuBot

        assert isinstance(bot, FeishuBot)
        if TYPE_CHECKING:
            assert isinstance(new, self.get_message_type())

        msg_type, content = new.serialize()
        params = {
            "method": "PUT",
            "body": {
                "content": content,
                "msg_type": msg_type,
            },
        }

        return await bot.call_api(f"im/v1/messages/{mid['message_id']}", **params)

    def get_reply(self, mid: Any):
        return Reply(mid["message_id"])



from typing import Union

import pytest
from nonebug import App
from nonebot import get_adapter
from arclet.alconna import Args, Alconna
from nonebot.adapters.onebot.v11 import Bot, Adapter, Message

from tests.fake import fake_group_message_event_v11


@pytest.mark.asyncio()
async def test_unmatch(app: App):
    from nonebot_plugin_alconna import At, Match, UniMessage, on_alconna

    test_cmd = on_alconna(
        Alconna("test", Args["target", Union[int, At]]), skip_for_unmatch=False, auto_send_output=True
    )

    @test_cmd.handle()
    async def tt_h(target: Match[Union[int, At]]):
        await test_cmd.send(UniMessage(["ok\n", str(target.result)]))

    async with app.test_matcher(test_cmd) as ctx:
        adapter = get_adapter(Adapter)
        bot = ctx.create_bot(base=Bot, adapter=adapter)

        event = fake_group_message_event_v11(message=Message("tes 1234"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_not_pass_rule()

        event = fake_group_message_event_v11(message=Message("test 1234"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_call_send(event, Message("ok\n1234"))

        event = fake_group_message_event_v11(message=Message("test abcd"), user_id=123)
        ctx.receive_event(bot, event)
        ctx.should_not_pass_rule()
        ctx.should_call_send(event, "ParamsUnmatched('参数 abcd 不正确')", bot=bot)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union, cast

from tarina import lang
from nonebot.internal.driver import Request
from nonebot.adapters import Bot, Event, Message

from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export
from ..segment import At, File, Text, AtAll, Audio, Emoji, Image, Reply, Video, Voice

if TYPE_CHECKING:
    from nonebot.adapters.discord.message import MessageSegment


class DiscordMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.discord.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.discord

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.discord.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return str(event.message_id)

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.discord.api.model import Channel
        from nonebot.adapters.discord.event import MessageEvent, GuildMessageCreateEvent

        if isinstance(event, MessageEvent):
            if isinstance(event, GuildMessageCreateEvent):
                return Target(
                    str(event.channel_id),
                    str(event.guild_id),
                    channel=True,
                    platform=self.get_adapter(),
                    self_id=bot.self_id if bot else None,
                )
            return Target(
                str(event.channel_id),
                channel=True,
                platform=self.get_adapter(),
                self_id=bot.self_id if bot else None,
            )
        elif isinstance(event, Channel):
            return Target(
                str(event.id), channel=True, platform=self.get_adapter(), self_id=bot.self_id if bot else None
            )
        raise NotImplementedError

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        if seg.flag == "role":
            return ms.mention_role(int(seg.target))
        elif seg.flag == "channel":
            return ms.mention_channel(int(seg.target))
        else:
            return ms.mention_user(int(seg.target))

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.mention_everyone()

    @export
    async def emoji(self, seg: Emoji, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.custom_emoji(seg.name or "", seg.id)

    @export
    async def media(self, seg: Union[Image, Voice, Video, Audio, File], bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        name = seg.__class__.__name__.lower()

        if seg.raw and (seg.id or seg.name):
            content = seg.raw_bytes
            return ms.attachment(seg.id or seg.name, content=content)
        elif seg.path:
            path = Path(seg.path)
            return ms.attachment(path.name, content=path.read_bytes())
        elif seg.url and (seg.id or seg.name):
            resp = await bot.adapter.request(Request("GET", seg.url))
            return ms.attachment(
                seg.id or seg.name,
                content=resp.content,  # type: ignore
            )
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.reference(seg.origin or int(seg.id), fail_if_not_exists=False)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.discord import Bot as DiscordBot

        assert isinstance(bot, DiscordBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        return await bot.send_to(channel_id=int(target.id), message=message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.discord.api.model import MessageGet
        from nonebot.adapters.discord.bot import Bot as DiscordBot

        _mid: MessageGet = cast(MessageGet, mid)

        assert isinstance(bot, DiscordBot)

        return await bot.delete_message(channel_id=mid.channel_id, message_id=_mid.id)

    async def edit(self, new: Message, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.discord.api.model import MessageGet
        from nonebot.adapters.discord.bot import Bot as DiscordBot
        from nonebot.adapters.discord.message import parse_message

        _mid: MessageGet = cast(MessageGet, mid)

        assert isinstance(bot, DiscordBot)
        if TYPE_CHECKING:
            assert isinstance(new, self.get_message_type())

        return await bot.edit_message(channel_id=mid.channel_id, message_id=_mid.id, **parse_message(new))

    def get_reply(self, mid: Any):
        from nonebot.adapters.discord.api.model import MessageGet

        _mid: MessageGet = cast(MessageGet, mid)
        return Reply(str(_mid.id), origin=_mid.id)



from pathlib import Path
from typing import TYPE_CHECKING, Any, Union, cast

from tarina import lang
from nonebot.internal.driver import Request
from nonebot.adapters import Bot, Event, Message

from ..export import Target, SupportAdapter, MessageExporter, SerializeFailed, export
from ..segment import At, File, Text, AtAll, Audio, Emoji, Image, Reply, Video, Voice, Reference, CustomNode

if TYPE_CHECKING:
    from nonebot.adapters.red.message import MessageSegment


class RedMessageExporter(MessageExporter["MessageSegment"]):
    def get_message_type(self):
        from nonebot.adapters.red.message import Message

        return Message

    @classmethod
    def get_adapter(cls) -> SupportAdapter:
        return SupportAdapter.red

    def get_target(self, event: Event, bot: Union[Bot, None] = None) -> Target:
        from nonebot.adapters.red.api.model import ChatType
        from nonebot.adapters.red.api.model import Message as MessageModel

        assert isinstance(event, MessageModel)
        return Target(
            str(event.peerUin or event.peerUid),
            private=event.chatType == ChatType.FRIEND,
            platform=self.get_adapter(),
            self_id=bot.self_id if bot else None,
        )

    def get_message_id(self, event: Event) -> str:
        from nonebot.adapters.red.event import MessageEvent

        assert isinstance(event, MessageEvent)
        return f"{event.msgId}#{event.msgSeq}"

    @export
    async def text(self, seg: Text, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        return ms.text(seg.text)

    @export
    async def at(self, seg: At, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.at(seg.target)

    @export
    async def at_all(self, seg: AtAll, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.at_all()

    @export
    async def emoji(self, seg: Emoji, bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        return ms.face(seg.id)

    @export
    async def media(self, seg: Union[Image, Video, File], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        method = {
            "image": ms.image,
            "video": ms.video,
            "file": ms.file,
        }[name]
        if seg.path:
            return method(Path(seg.path))
        elif seg.raw:
            return method(seg.raw_bytes)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            return method(resp.content)  # type: ignore
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def voice(self, seg: Union[Voice, Audio], bot: Bot) -> "MessageSegment":
        ms = self.segment_class

        name = seg.__class__.__name__.lower()
        if seg.path:
            return ms.voice(Path(seg.path), duration=seg.duration or 1)
        elif seg.raw:
            return ms.voice(seg.raw_bytes, duration=seg.duration or 1)
        elif seg.url:
            resp = await bot.adapter.request(Request("GET", seg.url))
            return ms.voice(resp.content, duration=seg.duration or 1)  # type: ignore
        else:
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type=name, seg=seg))

    @export
    async def reply(self, seg: Reply, bot: Bot) -> "MessageSegment":
        ms = self.segment_class
        if "#" in seg.id:
            _id, _seq = seg.id.split("#", 1)
            return ms.reply(_seq, _id)
        return ms.reply(seg.id)

    @export
    async def reference(self, seg: Reference, bot: Bot) -> "MessageSegment":
        from nonebot.adapters.red.message import ForwardNode

        ms = self.segment_class
        if not seg.content or not isinstance(seg.content, list):
            raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="forward", seg=seg))
        nodes = []
        for node in seg.content:
            if not isinstance(node, CustomNode):
                raise SerializeFailed(lang.require("nbp-uniseg", "invalid_segment").format(type="forward", seg=seg))
            content = self.get_message_type()()
            if isinstance(node.content, str):
                content.extend(self.get_message_type()(node.content))
            elif isinstance(node.content, list):
                content.extend(await self.export(node.content, bot, True))  # type: ignore
            else:
                content.extend(node.content)
            nodes.append(ForwardNode(uin=node.uid, name=node.name, time=node.time, message=content))
        return ms.forward(nodes)

    async def send_to(self, target: Union[Target, Event], bot: Bot, message: Message):
        from nonebot.adapters.red.bot import Bot as RedBot

        assert isinstance(bot, RedBot)
        if TYPE_CHECKING:
            assert isinstance(message, self.get_message_type())

        if isinstance(target, Event):
            target = self.get_target(target, bot)

        if target.private:
            return await bot.send_friend_message(target=target.id, message=message)
        else:
            return await bot.send_group_message(target=target.id, message=message)

    async def recall(self, mid: Any, bot: Bot, context: Union[Target, Event]):
        from nonebot.adapters.red.bot import Bot as RedBot
        from nonebot.adapters.red.api.model import Message as MessageModel

        assert isinstance(bot, RedBot)
        _mid: MessageModel = cast(MessageModel, mid)
        await bot.recall_message(_mid.chatType, _mid.peerUin, _mid.msgId)
        return

    def get_reply(self, mid: Any):
        from nonebot.adapters.red.api.model import Message as MessageModel

        _mid: MessageModel = cast(MessageModel, mid)
        return Reply(f"{_mid.msgId}#{_mid.msgSeq}")
